/* $Id: e2p_clone.c 1856 2006-04-24 12:51:28Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
//#include <unistd.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_task.h"
#include "e2_filelist.h"

/**
 * @brief copy selected item(s) each with a new name
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget optional pointer to widget where the action was initiated, generally NULL
 *
 * @return TRUE if action completed successfully, else FALSE
 */
static gboolean _e2p_clone (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	gchar *new_name, *slocal, *dlocal;
	GString *prompt = g_string_sized_new (NAME_MAX+64);
	GString *src = g_string_sized_new (PATH_MAX+NAME_MAX);
	GString *dest = g_string_sized_new (PATH_MAX+NAME_MAX);
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	guint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	//setup for tailoring over-write dialog
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	OW_ButtonFlags extras = (multisrc) ? NOALL : NONE;

	e2_filelist_disable_refresh ();
	e2_widget_set_cursor (GDK_WATCH);

	for (count=0; count < names->len; count++, iterator++)
	{
		g_string_assign (src, (*iterator)->filename);
		g_string_printf (prompt, "%s: <b>%s</b>", _("Enter name for copy of"), src->str);
		DialogButtons result = 0;
		e2_filelist_enable_refresh ();  //allow updates while we wait
		DialogButtons result2 = e2_dialog_line_input (_("clone"), prompt->str, src->str, extras, FALSE, &new_name);
		e2_filelist_disable_refresh ();
		if (result2 == OK)
		{
			g_string_printf (src, "%s%s", curr_view->dir, (*iterator)->filename);
			g_string_printf (dest, "%s%s", curr_view->dir, new_name);
			g_free (new_name);
			if (g_str_equal (src->str, dest->str)) continue;

			slocal = F_FILENAME_TO_LOCALE (src->str);
			dlocal = F_FILENAME_TO_LOCALE (dest->str);

			if (check && e2_fs_access2 (dlocal) == 0)
			{	//item with the new name already exists
				e2_filelist_enable_refresh ();   //allow updates while we wait
				result = e2_dialog_ow_check (slocal, dlocal, extras);
				e2_filelist_disable_refresh ();
				if (result == OK)
				{
					e2_task_do_task (E2_TASK_COPY, slocal, dlocal, FALSE, NULL);
				}
/*					else if (result == YES_TO_ALL)
				{
					do something smart about multiple-renames
				} */
				else if (result == NO_TO_ALL)
				{
					F_FREE (slocal);
					F_FREE (dlocal);
					break;
				}
			}
			else
			{
				e2_task_do_task (E2_TASK_COPY, slocal, dlocal, FALSE, NULL);
			}
			F_FREE (slocal);
			F_FREE (dlocal);
		}
		else if (result2 == NO_TO_ALL)
			break;
	}
	e2_fileview_clean_selected (names);
	g_string_free (prompt,TRUE);
	g_string_free (src,TRUE);
	g_string_free (dest,TRUE);

	//CHECKME E2_INCLIST - do current, & other view if it's same
	e2_filelist_check_dirty (GINT_TO_POINTER(1));

	e2_filelist_enable_refresh ();
	e2_widget_set_cursor (GDK_LEFT_PTR);
	return TRUE;
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "clone"
  aname = _("clone");

  p->signature = ANAME VERSION;
  p->menu_name = _("C_lone..");
  p->description = _("Copy selected item(s), each with new name, to the current directory");
  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
//  p->icon = "";  add icon file pathname if appropriate

  if (p->plugin_cb == NULL)
  {
	p->plugin_cb = _e2p_clone;  //set plugins menu item function
	//no need to free this
	gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
	e2_plugins_action_register (action_name, E2_ACTION_TYPE_ITEM, _e2p_clone,
	  NULL, FALSE, 0, NULL);
	return TRUE;
  }
  return FALSE;
}
/**
 * @brief cleanup transient things for this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return TRUE if all cleanups were completed
 **/
gboolean clean_plugin (Plugin *p)
{
  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
  gboolean ret = e2_plugins_action_unregister (action_name);
  g_free (action_name);
  return ret;
}
