/* $Id: e2p_find.h 1590 2006-02-09 06:11:03Z tpgww $
 *
 * Copyright  (C) 2005 tooar <tooar@gmx.net>
 * Portions copyright  (C) 1999 Matthew Grossman <mattg@oz.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
// this code was originally sourced from gtkfind 1.1 by Matthew Grossman
/**
 * @file plugins/e2p_find.h 
 * @brief header for file-find plugin
 *
 **/

#ifndef E2P_FIND_MAIN_H
# define E2P_FIND_MAIN_H

#include "emelfm2.h"

//support mime-type search criteria
#define MIMEFIND

typedef enum
	{	SEARCH_SUBDIRS_P = 0,
		STRING_FILENAME_P, WILDCARD_FILENAME_P, REGEXP_FILENAME_P, ANYCASE_FILENAME_P,
		//these used for shell cmds only
//		SHELL_COMMAND_P,
//		PRINT_TO_STDOUT_P, PRINT_TO_WINDOW_P, PRINT_FILENAME_ANYWAY_P,

//		WARNING_WINDOW_P,
//		LONG_OUTPUT_P, SHORT_OUTPUT_P,

		STRING_CONTENT_P, WILDCARD_CONTENT_P, REGEXP_CONTENT_P, ANYCASE_CONTENT_P,
		
		MODE_IS_P, MODE_OR_P, MODE_NOT_P,
//for decoding, these need to be in the same order as the flags in the permissions dialog
//"u+s", "g+s", "o+t", "u+r", "u+w", "u+x", "g+r", "g+w", "g+x", "o+r", "o+w", "o+x"
		SETUID_P, SETGID_P, STICKY_P,
		OWNER_READ_P, OWNER_WRITE_P, OWNER_EXEC_P,
		GROUP_READ_P, GROUP_WRITE_P, GROUP_EXEC_P,
		WORLD_READ_P, WORLD_WRITE_P, WORLD_EXEC_P, 

		REGULAR_P, DIRECTORY_P, RAW_DEVICE_P, BLOCK_DEVICE_P, 
		SYMLINK_P, SOCKET_P, FIFO_P, 

		UID_ANY_P, UID_SPECIFIC_P, UID_NONE_P, UID_LOGIN_P, UID_NOT_LOGIN_P,
		GID_ANY_P, GID_SPECIFIC_P, GID_NONE_P, GID_LOGIN_P, GID_NOT_LOGIN_P,

		FSIZE_LT_P, FSIZE_EQ_P, FSIZE_GT_P, 
		FSIZE_B_P, FSIZE_KB_P, FSIZE_MB_P, 

		MTIME_ET_P, MTIME_EQ_P, MTIME_LT_P,
		ATIME_ET_P, ATIME_EQ_P, ATIME_LT_P, 
		CTIME_ET_P, CTIME_EQ_P, CTIME_LT_P, 

		MAX_FLAGS	//no. of entries in the array
	} flag_t ;
//this is the value which starts the permissions flags
//used for decoding the flags to a mode number
#define PERMISSIONS1 SETUID_P

typedef struct
{
	GtkWidget *day_spin;
	GtkWidget *month_spin;
	GtkWidget *year_spin;
	GtkWidget *hour_spin;
	GtkWidget *minute_spin;
} spinners;
	
typedef struct
{
	GtkWidget *dialog;
	GtkWidget *notebook; // notebook, gui for the various search criteria
	GtkWidget *directory; // entry with text for directory to search in/from
	GtkWidget *pattern;  // entry with text for file name to search for
#ifdef MIMEFIND
	GtkWidget *mime_entry;	// entry with user-specified mimetype
#endif
	GtkWidget *content_pattern; // entry with text for file content
	//GtkWidget *shell_command; // if we run a shell command, this is it
	GtkWidget *curr_user;	//
	GtkWidget *choose_user;	//
	GtkWidget *user_entry;	// entry with user-specified uid
	GtkWidget *curr_group;	//
	GtkWidget *choose_group;	//
	GtkWidget *group_entry;	// entry with user-specified uid
	GtkWidget *size_entry;	// entry with user-specified filesize
	//spin buttons for file date/time parameters
	spinners mtime;
	spinners atime;
	spinners ctime;
//	GtkWidget *stop_button;  // button widget, remebered for changing sensitivity
//	GtkWidget *start_button;  // ditto
//	GtkWidget *help_button;  // ditto
//	gboolean stop_flag;	//TRUE when stop button clicked
	GSList *groups;  //list of lists of grouped toggle btns
	gint find_pid;
//now static	gboolean flags[MAX_FLAGS];	//cache for toggle values
} E2_FindDialogRuntime;

typedef struct
{
	gfloat day;
	gfloat month;
	gfloat year;
	gfloat hour;
	gfloat minute;
//	gfloat second;
} local_dt;

typedef struct
{
	glong days;
	gint minutes;
} offset;

//extern void find (gpointer *find_button, gpointer *pattern);

void e2pf_make_directory_widgets (GtkWidget *parent, E2_FindDialogRuntime *rt);
#ifdef USE_SHELL
void e2pf_make_shell_widgets (GtkWidget *parent);
#endif
void e2pf_make_notebook (GtkWidget *parent, E2_FindDialogRuntime *rt);
void e2pf_make_filename_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
void e2pf_make_filesize_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
#ifdef MIMEFIND
void e2pf_make_mimetype_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
#endif
void e2pf_make_content_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
void e2pf_make_atime_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
void e2pf_make_ctime_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
void e2pf_make_mtime_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
void e2pf_make_mode_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
void e2pf_make_type_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);
void e2pf_make_owner_tab (GtkWidget *notebook, E2_FindDialogRuntime *rt);

#define e2pf_create_vbox(box) e2_widget_add_box(box,TRUE,0,TRUE,FALSE,E2_PADDING)
#define e2pf_create_hbox(box) e2_widget_add_box(box,TRUE,0,FALSE,FALSE,E2_PADDING)
#define e2pf_create_label(box,text) e2_widget_add_label(box,text,0.0,TRUE,E2_PADDING_XSMALL)
#define e2pf_create_entry(box) e2_widget_add_entry(box,"",TRUE,FALSE)
//GtkWidget *e2pf_create_vbox (GtkWidget *container);
//GtkWidget *e2pf_create_hbox (GtkWidget *container);
//GtkWidget *e2pf_create_label (GtkWidget *container, gchar *text);
//GtkWidget *e2pf_create_entry (GtkWidget *container);
//GtkWidget *e2pf_create_button (GtkWidget *container, 
//	void  (*callback) (), gpointer callback_data);
GtkWidget *e2pf_create_toggle_grouped_button (GtkWidget *container, 
	flag_t f, gboolean state, gchar *label, GtkWidget *leader,
	E2_FindDialogRuntime *rt);
GtkWidget *e2pf_create_toggle_button (GtkWidget *container, 
	flag_t f, gboolean state, gchar *label, E2_FindDialogRuntime *rt);
GtkWidget *e2pf_create_radio_button (GtkWidget *container, 
	flag_t f, gchar *label, E2_FindDialogRuntime *rt);
GtkWidget *e2pf_create_radio_grouped_button (GtkWidget *container, GtkWidget *group, 
	flag_t f, gboolean state, gchar *label, E2_FindDialogRuntime *rt);
/*GtkWidget *e2pf_create_filesel (gchar *title, 
	void (*ok)(GtkWidget *, E2_FindDialogRuntime *),
	void(*cancel)(GtkWidget *, E2_FindDialogRuntime *), 
	E2_FindDialogRuntime *rt); */
GtkWidget *e2pf_create_spin_button (gfloat *default_value, gfloat min_value,
	gfloat max_value);

gboolean e2pf_check_leapyear (gint year);
void e2pf_set_flag (flag_t f, gboolean value, E2_FindDialogRuntime *rt);
gboolean e2pf_get_flag (flag_t f, E2_FindDialogRuntime *rt);
void e2pf_reset_flags (E2_FindDialogRuntime *rt);
void e2pf_get_relative_datetime (GDate *now, offset *result, spinners *times);
void e2pf_get_current_datetime (local_dt *current);
void e2pf_make_all_spinners (GtkWidget *parent, spinners *times);
void e2pf_reset_all_widgets (GtkWidget *widget, gpointer data);
void reset_entry (GtkWidget *widget);
void e2pf_reset_spin_button (GtkWidget *widget);
void e2pf_set_toggle_button_off (GtkWidget *widget);
void e2pf_set_toggle_button_on (GtkWidget *widget);

#endif //ndef E2P_FIND_MAIN_H
