/* $Id: e2p_sort_by_ext.c 1856 2006-04-24 12:51:28Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_plugins.h"
#include "e2_filelist.h"

extern gint stored_col_order[2][MAX_COLUMNS];
extern gint displayed_col_order[2][MAX_COLUMNS];

//FIXME = support case-insensitive sorting
gboolean first_time = TRUE;
//this is toggled each time the fn is called
gboolean order;  //TRUE for ascending sorts
gint ext_sort
	(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data)
{
	gchar *namea, *nameb, *ea, *eb;
	FileInfo *infoa, *infob;
	gint result;
	gtk_tree_model_get (model, a, FILENAME, &namea, FINFO, &infoa, -1);
	gtk_tree_model_get (model, b, FILENAME, &nameb, FINFO, &infob, -1);
	//all result codes are set for ascending order, toggled at end if need be
	if (g_str_equal (infoa->filename, ".."))
		result = (order) ? -1 : 1;
	else
		if (g_str_equal (infob->filename, ".."))
			result = (order) ? 1 : 1;
	else
		if (e2_fs_is_dir (infoa))
		{
			if (e2_fs_is_dir (infob))
				result = g_utf8_collate (namea, nameb);
			else
		  		result = (order) ? -1 : 1;
		}
	else
		if (e2_fs_is_dir (infob))
			result = (order) ? 1 : -1;
	else
	{
		if ((ea = strrchr (namea, '.')) == NULL)	//always ascii '.', don't need g_utf8_strrchr()
		{	// row a has NO extension.. check row b
			if ((strrchr (nameb, '.')) != NULL)
				result = -1;
			else
				result = (order) ? g_utf8_collate (namea, nameb) :
					g_utf8_collate (nameb, namea);
		}
		// row a HAS an extension.. check row b
		else
			if ((eb = strrchr (nameb, '.')) == NULL)
				result = 1;
		// both have extensions.. see if they are the same
		else
			if ((result = g_utf8_collate (ea, eb)) == 0)
				result = (order) ? g_utf8_collate (namea, nameb) :
					g_utf8_collate (nameb, namea);
	}
	g_free (namea);
	g_free (nameb);
	return result;
}
/**
 * @brief sort active filepane by item-extension
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget optional pointer to widget where the action was initiated, generally NULL
 *
 * @return TRUE if action completed successfully, else FALSE
 */
static gboolean _e2p_sort_by_ext (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	//temporarily replace the sort function for filename column of liststore
	GtkTreeModel *mdl = GTK_TREE_MODEL (curr_view->store);
    GtkTreeSortable *sortable = GTK_TREE_SORTABLE (mdl);
	gint sortcolnow;
	GtkSortType sort_order;

	if (first_time)
	{
		printd (DEBUG, "first extension sort");
		order = FALSE;
		first_time = FALSE;
	}
	order = !order;  //toggle local direction flag

	gtk_tree_sortable_get_sort_column_id (sortable, &sortcolnow, &sort_order);
	gtk_widget_hide (curr_view->sort_arrows[sortcolnow]);
	//pick some different arrow form as a visual indicator
	gtk_arrow_set (GTK_ARROW (curr_view->sort_arrows[FILENAME]),
		GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
	gtk_widget_show (curr_view->sort_arrows[FILENAME]);

	gtk_tree_sortable_set_sort_func (sortable, FILENAME, ext_sort, NULL, NULL);
	sort_order = (order) ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING;
	//do the sort
	gtk_tree_sortable_set_sort_column_id (sortable, FILENAME, sort_order);
	return TRUE;
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "sort_by_ext"
  aname = _("sort_by_ext");

  p->signature = ANAME VERSION;
  p->menu_name = _("Extension _sort");
  p->description = _("Sort the active file pane by filename extension");
  p->icon = "plugin_extsort_"E2IP".png";  //use icon file pathname if appropriate

  if (p->plugin_cb == NULL)
  {
	p->plugin_cb = _e2p_sort_by_ext;  //set plugins menu item function
	//no need to free this
	gchar *action_name = g_strconcat (_A(9),".",aname,NULL);
	e2_plugins_action_register (action_name, E2_ACTION_TYPE_ITEM, _e2p_sort_by_ext,
	  NULL, FALSE, 0, NULL);
	return TRUE;
  }
  return FALSE;
}
/**
 * @brief cleanup transient things for this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return TRUE if all cleanups were completed
 **/
gboolean clean_plugin (Plugin *p)
{
  gchar *action_name = g_strconcat (_A(9),".",aname,NULL);
  gboolean ret = e2_plugins_action_unregister (action_name);
  g_free (action_name);
  return ret;
}
