/* $Id: e2p_view.c 1856 2006-04-24 12:51:28Z tpgww $
 *
 * Copyright (C) 2003-2006 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include "e2_plugins.h"
#include "e2_task.h"

/**
 * @brief view first selected item
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget optional pointer to widget where the action was initiated, generally NULL
 *
 * @return TRUE if action completed successfully, else FALSE
 */
static gboolean _e2p_view (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	return (e2_task_view (action_data, rt_data, widget));
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "view"
  aname = _("view_with_plugin");	//note: the action-name cannot be same as internal file.view

  p->signature = ANAME VERSION;
  p->menu_name = _("_View");
  p->description = g_strdup_printf (_("Send the first selected item to the %s text-file viewer"), PROGNAME);
//  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
//  p->icon = "";  add icon file pathname if appropriate

  if (p->plugin_cb == NULL)
 {
	  p->plugin_cb = _e2p_view;  //set plugins menu item function
	  //no need to free this
	  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
	  e2_plugins_action_register (action_name, E2_ACTION_TYPE_ITEM,
		 _e2p_view, NULL, FALSE, 0, NULL);
	  return TRUE;
  }
  return FALSE;
}
/**
 * @brief cleanup transient things for this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return TRUE if all cleanups were completed
 **/
gboolean clean_plugin (Plugin *p)
{
  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
  gboolean ret = e2_plugins_action_unregister (action_name);
  g_free (action_name);
  return ret;
}
