/* 
 * Copyright (C) 2004-2006 Florian Zaehringer <flo.zaehringer@web.de>
 * Copyright (C) 1999-2004 Bill Wilson <bill@gkrellm.net>
 *
 * This file is a contribution to emelFM2 written by tooar <tooar@gmx.net>
 * Functions dealing with fstab / mtab have been sourced from gkrellm 
 * (http://gkrellm.net) written by Bill Wilson.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "e2_complete.h"

#ifdef E2_FS_MOUNTABLE

#include <string.h>
#include <unistd.h>

#if defined(__linux__)
#include <mntent.h>
#include <fstab.h>

#elif defined(__E2BSD__)
#include <sys/param.h>
#include <sys/mount.h>
#include <fstab.h>

#elif defined(__solaris__) || defined(sco)
//CHECKME sco ?? svr5 in general ??
#include <sys/mnttab.h>
#include <vfstab.h>

#else
#include <mntent.h>
#include <fstab.h>

/*Some systems use statfs() to provide information about mounted
file systems, other systems use statvfs(). The header files used with
these functions vary between systems.
e.g.
HPUX 	  statfs() with sys/vfs.h
SunOS   statfs() with sys/vfs.h.
Solaris  statvfs() with sys/statvfs.h. */

#include <sys/types.h>
#include <sys/param.h>
#ifdef HAVE_SYS_STATVFS_H
#include <sys/statvfs.h>
#endif
#ifdef HAVE_SYS_VFS_H
#include <sys/vfs.h>
#endif

/*#ifdef HAVE_SYS_STATVFS_H
#define STATFS statvfs
#else
#define STATFS statfs
#endif

...
     struct STATFS sfs;
...
     rc = STATFS(filename, &sfs);
*/
#endif	//which os-specifis includes

//test which mounted devices should be included in the list
//of unmountable ones
//CHECKME does this need to be os-specific ? if so, how ?
#define E2_TEST_MOUNTED_DEVICE \
		if (     strcmp (dev, "none") \
			&& strcmp (dev, "/dev") \
			&& strcmp (dir, G_DIR_SEPARATOR_S)) /*no need to support umounting '/' */ \
/*	these should be handled by dev="none"
			&& strcmp (type, "devpts")
			&& strcmp (type, "proc")
			&& strcmp (type, "usbdevfs")
			&& strcmp (type, "usbfs")
			&& strcmp (type, "sysfs")
			&& strstr (dir, "/dev/") */
/* permission-checking disabled pending doing it properly, if not forever
			gchar *opt = mounted->mnt_opts;
			if (!permcheck || _e2_complete_mount_permission (dir, opt)) */
/*for bsd's, which mount options are relevant ? how do we check them ?
		mounted[i].f_flags	//mount flags
		mounted[i].f_owner	//owner
*/
/* there may be things blocking an unmount eg monitoring with dnotify */

//finesse mounted device name before inclusion in the list of
//unmountable devices
#define E2_PROCESS_MOUNTED_DEVICE
//strip any trailing / from the mountpoint name
//			s = strrchr (dir, G_DIR_SEPARATOR);
//			if (s != NULL && s != dir && *(s+1) == '\0')
//				*s = '\0';

//test which unmounted devices should be included in the list
//of mountable ones
//CHECKME does this need to be os-specific ? if so, how ?
#define E2_TEST_UNMOUNTED_DEVICE \
		if (      strcmp (dev, "none") \
			&& strcmp (type, "swap") \
			&& strcmp (type, "ignore") \
			&& strcmp (dir, G_DIR_SEPARATOR_S)	/*no need to support mounting '/' */ \
			&& type[0] != '\0' )
/*
//			&& g_file_test (dir, G_FILE_TEST_IS_DIR) don't bother checking for this (slow)
    these should be handled by dev="none"
			&& strcmp (type, "devpts")
			&& strcmp (type, "proc")
			&& strcmp (type, "sysfs")
			&& strcmp (type, "usbdevfs")
			&& strcmp (type, "usbfs")
			&& strstr (dir, "/dev/") == NULL
CHECKME sometimes, test may need to include
	strncmp (dev, "/dev", 4)
	realpath (dev, realdev) != NULL
	gchar realdev[PATH_MAX];
*/
/* permission-checking disabled pending doing it properly, if not forever
			gchar *opt = fs.vfs_mntopts;
			if (!permcheck || _e2_complete_mount_permission (dir, opt)) */

//finesse unmounted device name before inclusion in the list of
//mountable devices
#define E2_PROCESS_UNMOUNTED_DEVICE
			//strip any trailing / from the mountpoint name
//			s = strrchr (dir, G_DIR_SEPARATOR);
//			if (s != NULL && s != dir && *(s+1) == '\0')
//				*s = '\0';

//guint uid;
//time_t fstab_mtime;

/**
 * @brief add mountpoint @a dir to list of points @a mounts
 * The string is copied, with conversion to utf8 if need be
 * @param dir localised string with path of mountpoint to be added
 * @param points pointer to the list to be updated
 * @return
 **/
static void _e2_complete_mount_add_to_list (gchar *dir, GList **points)
{
	gchar *utf = D_FILENAME_FROM_LOCALE (dir);
	*points = g_list_append (*points, utf);
}

/**
 * @brief replace escaped character codes in @a buf
 * @param buf buffer with string from fs/mtab, presumably locale-encoded
 * @return
 **/
/* UNUSED
static void _e2_complete_mount_fix_fstab_name (gchar *buf)
{
	gchar *rp, *wp;

	if (buf[0] == '\0')
		return;
	rp = buf;
	wp = buf;
	do	// This loop same as in libc6 getmntent()
		if (rp[0] == '\\' && rp[1] == '0' && rp[2] == '4' && rp[3] == '0')
		{
			*wp++ = ' ';		// \040 is a SPACE
			rp += 3;
		}
		else if (rp[0] == '\\' && rp[1] == '0' && rp[2] == '1' && rp[3] == '2')
		{
			*wp++ = '\t';		// \012 is a TAB
			rp += 3;
		}
		else if (rp[0] == '\\' && rp[1] == '\\')
		{
			*wp++ = '\\';		// \\ is a \
			rp += 1;
		}
		else
			*wp++ = *rp;
	while (*rp++ != '\0');
} */
/**
 * @brief check whether current user is permitted to [u]mount mountpoint @a device
 * @param device mountpoint path-string, presumably locale-encoded
 * @param options is a string with the (4th) parameters string from a line in fs/mtab 
 * @return TRUE if permitted
 **/
/* UNUSED FIXME, if this is used, needs to account for sudo & group membership etc
 and work properly for all os's
static gboolean _e2_complete_mount_permission (gchar* device, gchar *options)
{
	if (uid == 0 	//root can always do it
		|| strstr (options, "dev,") != NULL //CHECKME _I(
		|| (strstr (options, "user") != NULL && strstr (options, "nouser") == NULL)) //CHECKME _I(
		return TRUE;
#if defined (__linux__) 
	struct stat my_stat;
	//no need to localise *device before stat
	return (e2_fs_stat (device, &my_stat) == 0
		&& my_stat.st_uid == uid 
		&& strstr (options, "owner") != NULL //CHECKME _I(
	   );
#else
	return FALSE;
#endif
} */
/**
 * @brief create list of mounted partitions
  * The list is created by the appropriate os-specifc protocol
 * @param mounts_list pointer to list for storing results
 * @return
 **/
 static void _e2_complete_mount_get_mounts_list (GList **mounts_list)	//gboolean permcheck)
{
	//FIXME lock the data source while accessing it
#if defined(__linux__)
	struct mntent *mounted;
	E2_FILE *f;
/* When the linux proc filesystem is mounted, the files /etc/mtab and /proc/mounts
	have very similar contents. The former has somewhat more information (CHECKME),
	such as the mount options used, but is not necessarily up-to-date */
	if ((f = setmntent ("/proc/mounts", "r")) == NULL)
		if ((f = setmntent ("/etc/mtab", "r")) == NULL)
			return;

	while ((mounted = getmntent (f)) != NULL)
	{
		//check the mounted device meets our needs
		//need to take a copy if string is altered
		gchar *dev = mounted->mnt_fsname;
		gchar *dir = mounted->mnt_dir;
//		gchar *type = mounted->mnt_type;
		E2_TEST_MOUNTED_DEVICE
		{
			E2_PROCESS_MOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, mounts_list);
		}
	}
	endmntent (f);
#elif defined(__E2BSD__)
	struct statfs *mounted;

	//NOTE: data provided by this is not process- or thread-safe
	gint i, count = getmntinfo (&mounted, MNT_NOWAIT);

	for (i=0; i<count; i++)
	{
		//check the mounted device meets our needs
		//need to take a copy if string is altered
		gchar *dev = mounted[i].f_mntfromname;
		gchar *dir = mounted[i].f_mntonname;
//		gchar *type = mounted[i].f_fstypename;
		E2_TEST_MOUNTED_DEVICE
		{
			E2_PROCESS_MOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, mounts_list);
		}
	}
#elif defined(__solaris__) || defined(sco)
	//CHECKME sco?
	struct mnttab mounted;
	E2_FILE *f;

	if ((f = fopen("/etc/mnttab", "r")) == NULL)
		return;
	
	while (getmntent (f, &mounted) == 0)
	{
		//check the mounted device meets our needs
		//need to take a copy if string is altered
		gchar *dev = mounted.mnt_special;
		gchar *dir = mounted.mnt_mountp;
//		gchar *type = mounted.mnt_fstype;
		E2_TEST_MOUNTED_DEVICE
		{
			E2_PROCESS_MOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, mounts_list);
		}
	}
	fclose (f);
#elif defined(hpux)
	struct mntent *mounted;
	E2_FILE *f;

	const gchar *mtab = (e2_fs_access ("/etc/pfs_mtab", F_OK)) ?
		"/etc/mtab" : "/etc/pfs_mtab";
	if ((f = setmntent (mtab, "r")) == NULL)
		return;

	while ((mounted = getmntent (f)) != NULL)
	{
		//check the mounted device meets our needs
		//need to take a copy if string is altered
		gchar *dev = mounted->mnt_fsname;
		gchar *dir = mounted->mnt_dir;
//		gchar *type = mounted->mnt_type;
		E2_TEST_MOUNTED_DEVICE
		{
			E2_PROCESS_MOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, mounts_list);
		}
	}
	endmntent (f);
#elif defined(__svr4__)
	struct mnttab mounted;
	E2_FILE *f;

	if ((f = fopen ("/etc/mtab", "r") == NULL)
		return;

	while (getmntent (f, &mounted) ==0)
	{
		gchar *dev = mounted.f_mntfromname;
		gchar *dir = mounted.f_mntonname;
//		gchar *type = mounted.f_fstypename;
		E2_TEST_MOUNTED_DEVICE
		{
			E2_PROCESS_MOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, mounts_list);
		}
	}
	fclose (f);
#else
//other unix-like os's do it like linux, but without /proc/mount
//CHECKME sometimes with a different filename ??
	struct mntent *mounted;
	E2_FILE *f;

	if ((f = setmntent ("/etc/mtab", "r")) == NULL)
		return;

	while ((mounted = getmntent (f)) != NULL)
	{
		//check the mounted device meets our needs
		//need to take a copy if string is altered
		gchar *dev = mounted->mnt_fsname;
		gchar *dir = mounted->mnt_dir;
//		gchar *type = mounted->mnt_type;
		E2_TEST_MOUNTED_DEVICE
		{
			E2_PROCESS_MOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, mounts_list);
		}
	}
	endmntent (f);
#endif	//which OS
}
/**
 * @brief create list of mountable partitions
 * The list is created by the appropriate os-specifc protocol
 * @param fstab_list pointer to list for storing results
 * @return
 **/
static void _e2_complete_mount_get_mountable_list (GList **fstab_list)	//gboolean permcheck)
{
	//CHECKME lock data file while accessing it ??
#if defined(__E2BSD__) || defined(__linux__)
    struct fstab *fs;

	if (!setfsent ())
		return;

	while ((fs = getfsent ()) != NULL)
	{
		gchar *dev = fs->fs_spec;
		gchar *dir = fs->fs_file;
		gchar *type = fs->fs_vfstype;
/*		copy strings if they need to be modified
		g_strlcpy (dev, fs->fs_spec, sizeof(dev));
		g_strlcpy (dir, fs->fs_file, sizeof(dir));
		g_strlcpy (type, fs->fs_vfstype, sizeof(type));
		_e2_complete_mount_fix_fstab_name (dev);
		_e2_complete_mount_fix_fstab_name (dir);
		_e2_complete_mount_fix_fstab_name (type); */
		E2_TEST_UNMOUNTED_DEVICE
		{
			E2_PROCESS_UNMOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, fstab_list);
		}
    }
    endfsent ();
#elif defined(__solaris__) || defined(sco)
	//CHECKME sco
    struct vfstab fs;
	E2_FILE *f;

	if ((f = fopen ("/etc/vfstab", "r")) == NULL)
		return;

	while (getvfsent (f, &fs) == 0)
	{
		gchar *dev = fs.vfs_special;
		gchar *dir = fs.vfs_mountp;
//		gchar *type = fs.vfs_fstype;
		E2_TEST_UNMOUNTED_DEVICE
		{
			E2_PROCESS_UNMOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, fstab_list);
		}
	}
	fclose (f);
#elif defined(hpux)
    struct mntent *mountable;
	E2_FILE *f;

    //handle PFS if necessary
    const gchar *fstab = (access("/etc/pfs_fstab", F_OK)) ?
		"/etc/fstab" : "/etc/pfs_fstab";

	if ((f = setmntent (fstab, "r")) == NULL)
		return;

	while ((mountable = getmntent (f)) != NULL)
	{
		gchar *dev = mountable->mnt_fsname;
		gchar *type = mountable->mnt_type;
		E2_TEST_UNMOUNTED_DEVICE
		{
			E2_PROCESS_UNMOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, fstab_list);
		}
	}
	endmntent (f);
#elif defined(__svr4__)
	struct mnttab mountable;
	E2_FILE *f;

	if ((f = fopen ("/etc/fstab", "r")) == NULL)
		return;

	while (getmntent (f, &mountable) == 0)
	{
		gchar *dev = mountable.mnt_special;
		gchar *dir = mountable.mnt_mountp;
		E2_TEST_UNMOUNTED_DEVICE
		{
			E2_PROCESS_UNMOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, fstab_list);
		}
	}
	fclose (f);
#else
	struct mntent *mountable;
	E2_FILE *f;

	if ((f = setmntent ("/etc/fstab", "r" )) == NULL)
		return;

	while ((mountable = getmntent (f)) != NULL)	//data is not thread- or process-safe
	{
		gchar *dev = mountable->mnt_fsname;
		gchar *dir = mountable->mnt_dir;
//		gchar *type = mountable->mnt_type;
//		gchar *opts = mountable->mnt_opts;
		E2_TEST_UNMOUNTED_DEVICE
		{
			E2_PROCESS_UNMOUNTED_DEVICE
			_e2_complete_mount_add_to_list (dir, fstab_list);
		}
	}
	endmntent (f);
#endif	//which OS
}
/**
 * @brief construct list of the items in @a fs_list that complete @a word
 * 
 * @param fs_list list of items to be scanned
 * @param found pointer to list to hold matching items from @a fs_list
 * @param word utf-8 string with string to be completed ("mount" or start of mountpoint path)
 * @return the no. of matches found
 **/
static gint _e2_complete_mount_find_in_fslist (GList *fs_list,
	GList **found, gchar *word)
{
	guint i = 0;
	gint ret = 0;
	gboolean all = g_str_has_suffix (word, "mount");	//_I( the mountpoint was not started in the command
	gchar *path = NULL;
	if (!all)
	{
		if (g_str_has_prefix (word, "./"))
			path = g_strconcat (curr_pane->path, word+2, NULL);
		else
			path = g_strdup (word);
	}
	gchar *directory;

	for (i = 0; i < g_list_length (fs_list); i++)
	{
		gchar *dir = (gchar *) g_list_nth_data (fs_list, i);
		if (all || g_str_has_prefix (dir, path))
		{	//to conform to e2 'style', make sure all mountpoints have a trailing '/'
//			if (g_str_has_suffix (dir, G_DIR_SEPARATOR_S))
				directory = g_strdup (dir);
//			else
//				directory = g_strconcat (dir, G_DIR_SEPARATOR_S, NULL);

			if (!all)
			{
				gchar **dir_last_part = g_strsplit (directory, path, 2);
				g_free (directory);

				//prepend word, coz it got killed in g_strsplit
				directory = g_strconcat (word, dir_last_part[1], NULL);
				g_strfreev (dir_last_part);
			}

			*found = g_list_append (*found, directory);
			ret++;
			//if this matches - we take it. no doubled entries.
			continue;
		}

		if (g_str_has_prefix (dir, word))
		{
//			if (g_str_has_suffix (dir, G_DIR_SEPARATOR_S))
				directory = g_strdup (dir);
//			else
//				directory = g_strconcat (dir, G_DIR_SEPARATOR_S, NULL);

			*found = g_list_append (*found, directory);
			ret++;
		}
	}
	if (path != NULL)
		g_free (path);
	return ret;
}
/**
 * @brief for a mount or umount command, create list of partitions that can be mounted or unmounted
 *
 * The string to be completed extends from after the ' ' (if any) preceding
 * the cursor position @a pos, to that position. But if @a pos = 0, the string
 * will be empty
 *
 * @param line utf8 string containing the whole line to be completed
 * @param word utf8 string, copy of part of @a line with the 'word' to be completed
 * @param pos 0-based index of cursor position (characters, not bytes) in @a line
 * @param found pointer to list to record matching items
 * @param flags pointer to bit-flags indicating type of match
 * @param data discretionary data pointer for this method
 * @param rt UNUSED pointer to data struct for the command line whose entry is to be completed
 * @return the number of successful matches
 **/
gint e2_complete_mount (gchar *line, gchar *word, gint pos,
	GList **found, E2_CompleteFlags *flags, gpointer data, E2_CommandLineRuntime *rt)
{
	//make sure this is an (un)mount command
	gchar *tmp = g_utf8_offset_to_pointer (line, pos);
	gint byte_pos = (tmp - line);
	if (g_strstr_len (line, byte_pos, "mount") == NULL)	//_I( ??
		return 0;

	gint ret = 0;
	*flags |= E2_COMPLETE_FLAG_STOP;
//	uid = getuid (); needed if checking permissions

	if (g_strstr_len (line, byte_pos, "umount") != NULL)	//_I( ??
	{
		GList	*mounts_list = NULL;
		_e2_complete_mount_get_mounts_list (&mounts_list);
		ret = _e2_complete_mount_find_in_fslist (mounts_list, found, word);
		e2_list_free_with_data (&mounts_list);
	}
	else
	{
		GList	*fstab_list = NULL;
		_e2_complete_mount_get_mountable_list (&fstab_list);
		ret = _e2_complete_mount_find_in_fslist (fstab_list, found, word);
		e2_list_free_with_data (&fstab_list);
	}

	return ret;
}

/**
 * @brief handle a selection from a mountpoints menu
 * The state of @a item when it arrives here is opposite to that
 * shown in the menu, when clicked
 * @param item the selected menu item
 * 
 * @return
**/
static void _e2_complete_mount_menu_cb (GtkWidget *item)
{
	gchar *cmd;
//	gchar *olddir;
	gchar *point = g_object_get_data (G_OBJECT (item), "_mountpoint_");
	gboolean newstate = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item));
	if (!newstate)
	{	//doing an unmount, move out of current dir if necessary
		//so we don't ourself block the ummount
		if (g_str_has_prefix (curr_view->dir, point))
		{
			//char olddir = g_strdup (curr_view->dir;	//FIXME wrong place to do this
			gchar *s = strrchr (point, G_DIR_SEPARATOR);
			if (s > point+1)	//not doing root dir
			{
				*s = '\0';
				//change to an ancestor path if we can
				cmd = g_strconcat ("cd ", point, NULL);
				e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT);
				g_free (cmd);
				*s = G_DIR_SEPARATOR;
			}
		}
	}
	cmd = g_strconcat ((newstate) ? "mount" : "umount",	//CHECKME _i(
		" ", point, NULL);
//	gint result =
	e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT);
	g_free (cmd);
/*	if (!(newstate || result == 0))
	{	//change back to where we started
		cmd = g_strconcat ("cd ", olddir, NULL);
		e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT);
		g_free (cmd);
		g_free (olddir);
	} */
}
/**
 * @brief add check item to @a menu
 *
 * Can't use e2_menu_add_check() because we don't want mnemonics
 *
 * @param menu menu widget
 * @param label menu item text
 * @param state initial state of the created item, T/F
 * @param func void* callback function for handling menu item selection, or NULL
 *
 * @return the menu item widget
 **/
static GtkWidget *_e2_complete_mount_add_check 
	(GtkWidget *menu,  gchar *label, gboolean state, void *func)
{
	GtkWidget *check = gtk_check_menu_item_new_with_label (label);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), check);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (check), state);
	g_signal_connect (G_OBJECT (check), "toggled", G_CALLBACK (func), NULL);
	gtk_widget_show (check);
	return check;
}
/**
 * @brief create and pop up a mountpoints menu
 *
 * @param action_data ptr to data assigned when action struct created at session start NULL
 * @param arg argument string from the command which was executed
 * @param button the button that was clicked to popup the menu
 *
 * @return TRUE if the action succeeded
 */
gboolean e2_complete_mount_menu_create (gpointer action_data, 
	gchar *arg, GtkWidget *button)
{
	//determine which toolbar is being used
	if (arg == NULL || *arg == '\0')
		return FALSE;
	E2_ToolbarRuntime *rt;
	if (g_str_equal (arg, _C(3)))	//_("command bar")
		rt = &app.commandbar;
	else if (g_str_equal (arg, _C(39)))	//_("task bar"
		rt = &app.toolbar;
	else if (g_str_equal (arg, _C(29)))
		rt = &app.pane1.toolbar;
	else if (g_str_equal (arg, _C(31)))
		rt = &app.pane2.toolbar;
	else
		//give error message ?
		return FALSE;

	GtkWidget *menu = gtk_menu_new ();
	GtkWidget *item;

	GList *mounts = NULL;
	GList *mountables = NULL;
	_e2_complete_mount_get_mounts_list (&mounts);
	_e2_complete_mount_get_mountable_list (&mountables);
	GList *tmp, *tmp2;

	for (tmp = mounts ; tmp != NULL; tmp = g_list_next (tmp))
	{
		item = _e2_complete_mount_add_check (menu,
			(gchar *) tmp->data, TRUE, _e2_complete_mount_menu_cb);
		g_object_set_data_full (G_OBJECT (item), "_mountpoint_",
			g_strdup ((gchar *) tmp->data), g_free);
	}
	for (tmp = mountables ; tmp != NULL; tmp = g_list_next (tmp))
	{
		gchar *point = (gchar *) tmp->data;
		gboolean mounted = FALSE;
		for (tmp2 = mounts ; tmp2 != NULL; tmp2 = g_list_next (tmp2))
		{
			if (g_str_equal (point, (gchar *) tmp2->data))
			{
				mounted = TRUE;
				break;
			}
		}
		if (!mounted)
		{
			item = _e2_complete_mount_add_check (menu,
				(gchar *) tmp->data, FALSE, _e2_complete_mount_menu_cb);
			g_object_set_data_full (G_OBJECT (item), "_mountpoint_",
				g_strdup ((gchar *) tmp->data), g_free);
		}
	}

	e2_list_free_with_data (&mounts);
	e2_list_free_with_data (&mountables);

	g_signal_connect (G_OBJECT (menu), "selection-done",
		G_CALLBACK (e2_menu_destroy_cb), NULL);

	//determine the menu's popup position
	GtkAllocation menu_position;
	e2_utils_get_abs_pos (button, &(menu_position.x), &(menu_position.y));
	e2_toobar_menu_position_set (button, menu, &menu_position, rt);

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
		(GtkMenuPositionFunc) e2_toolbar_menu_position, &menu_position, 1, 0);

	return TRUE;
}

#endif	//def E2_FS_MOUNTABLE
