/* $Id: e2_mkdir_dialog.c 1861 2006-04-25 08:27:44Z tpgww $
 *
 * Copyright (C) 2004-2006 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
//#include <unistd.h>
#include <string.h>
#include "e2_dialog.h"
#include "e2_mkdir_dialog.h"
#include "e2_filelist.h"

static GList *history = NULL;
static gboolean _e2_mkdirdlg_changed_cb_idle (E2_MkdirDialogRuntime *rt);
static gboolean _e2_mkdirdlg_changed_cb (GtkWidget *entry, E2_MkdirDialogRuntime *rt);
static gboolean _e2_mkdirdlg_change_dir_cb (gchar *path, E2_MkdirDialogRuntime *rt);
static gboolean _e2_mkdirdlg_change_focus_cb (E2_PaneRuntime *pane_rt, E2_MkdirDialogRuntime *rt);

  /*****************/
 /***** utils *****/
/*****************/

static void _e2_mkdirdlg_dialog_destroy (E2_MkdirDialogRuntime *rt)
{
//#ifdef RACE_CHECK
//	printd (DEBUG, "mkdir dialog destroy");
//#endif
	gtk_widget_destroy (rt->dialog);
	e2_list_free_with_data (&rt->history);
	e2_hook_unregister (&app.pane1.hook_change_dir, _e2_mkdirdlg_change_dir_cb, rt, TRUE);
	e2_hook_unregister (&app.pane2.hook_change_dir, _e2_mkdirdlg_change_dir_cb, rt, TRUE);
	e2_hook_unregister (&app.hook_pane_focus_changed, _e2_mkdirdlg_change_focus_cb, rt, TRUE);
//#ifdef RACE_CHECK
//	printd (DEBUG, "mkdir dialog unhooks completed");
//#endif
	g_free (rt->path);
	if (rt->idle_id != 0)
#ifdef RACE_CHECK
	{
		printd (DEBUG, "mkdir dialog");
		gboolean debug =
#endif
		g_source_remove (rt->idle_id);
#ifdef RACE_CHECK
		if (debug)
			printd (DEBUG, " idle removal completed");
		else
			printd (DEBUG, " idle removal failed");
	}
//	else
//		printd (DEBUG, "mkdir dialog NO idle source to remove");
#endif
	g_free (rt);
//#ifdef RACE_CHECK
//	printd (DEBUG, "mkdir dialog destroy COMPLETED");
//#endif
}

static void _e2_mkdirdlg_update_dialog_size (E2_MkdirDialogRuntime *rt)
{
	if ((!GTK_IS_WIDGET (rt->dialog)) || (!GTK_WIDGET_VISIBLE (rt->dialog)))
		return;
	e2_dialog_resize_with_sw (rt->dialog, rt->scrolled);
}
/**
 * @brief determine a suggested directory name, by appending or incrementing a number
 *
 * @param dir last combo box entry, ie utf8, maybe NULL if no entry
 * @param parent_dir is rt->path, utf8
 * @return newly-allocated utf8 string
 */
static gchar *_e2_mkdirdlg_find_dir (const gchar *dir, const gchar *parent_dir)
{
	//we have to suggest something, but if there is nothing
	//in the history yet, we use a default string
	if (dir == NULL)
		dir = _("new directory");
	
	//the absolute directory name consists of 4 parts:
	//parent directory in case the entered string is not absolute
	gchar *part1 = g_path_is_absolute (dir) ?
		g_strdup ("") : g_strdup (parent_dir);
	//in case the user entered something like "dir/blah"
	gchar *part2 = g_path_get_dirname (dir);
	if (g_str_equal (part2, "."))
	{	//no path entered
		g_free (part2);
		part2 = g_strdup ("");
	}
	gchar *part3 = g_path_get_basename (dir);
	//last part to hold the rest of part 3 after we find a number
	gchar *part4 = "";
	//postion of the start of an existing number in part3
	gint digit_start = -1;
	//digit to increment or add to the directory name
	guint i = 0;
	//complete path (all utf8)
	gchar *path = g_build_filename (part1, part2, part3, part4, NULL);

	gboolean have_to_search = FALSE;
	//to simplify stuff, we do this check 2 times
	//this is the first time, if the directory the user
	//tries to create does already exist, it searches for a number
	//in part 3 and sets up part 4 (to the string after the number)
	gchar *local = F_FILENAME_TO_LOCALE (path);
	if (!e2_fs_access (local, F_OK))
	{
		have_to_search = TRUE;
		gint j = strlen (part3);
		//find first digit starting at the end of the string
		gboolean last_was_digit = FALSE;
		while (j >= 0)
		{
			if (g_ascii_isdigit (part3[j]))	//FIXME utf
			{
				if (!last_was_digit)
					last_was_digit = TRUE;
				if (j == 0)
					digit_start = 0;
			} else if (last_was_digit)
			{
				digit_start = j + 1;
				break;
			}
			j--;
		}
		if (digit_start != -1)
		{
			while (*(part3 + digit_start) == '0')
				digit_start++;
			if (!g_ascii_isdigit (*(part3 + digit_start)))
				digit_start--;
			i = g_ascii_strtoull (part3 + digit_start, &part4, 10);
			part3[digit_start] = '\0';
		}
	}
	//the real test loop that increments i
	while (!e2_fs_access (local, F_OK))
	{
		i++;
		g_free (path);
		if (digit_start != -1)
			path = g_strdup_printf ("%s"G_DIR_SEPARATOR_S"%s"G_DIR_SEPARATOR_S"%s%d%s",
				part1, part2, part3, i, part4);
		//if there is no digit/number in the directory string, we simply add it
		//to the end, ignoring part 4
		else
			path = g_strdup_printf ("%s"G_DIR_SEPARATOR_S"%s"G_DIR_SEPARATOR_S"%s %d",
				part1, part2, part3, i);
		F_FREE (local);
		local = F_FILENAME_TO_LOCALE (path);
	}
	g_free (path);
	F_FREE (local);
	path = NULL;
	
	//now build the string for the entry
	if (have_to_search)
	{
		//build the retval from the the string the user really entered
		//(dir argument)
		if (digit_start != -1)
		{
			path = g_strdup_printf ("%s%d%s", part3, i, part4);
			part3[digit_start] = '0';
		}
		else
			path = g_strdup_printf ("%s %d", part3, i);
	
		//add part2 if necessary
		if (!((part2[0] == '.') && (part2[1] == '\0')))
		{
			gchar *free = path;
			path = g_build_filename (part2, path, NULL);
			g_free (free);
		}
	}

	gchar *retval = (have_to_search) ? path : g_strdup (dir);
	
	//cleanup
	g_free (part1);
	g_free (part2);
	g_free (part3);
	if (!have_to_search)
		g_free (path);

	return retval;
}

static void _e2_mkdirdlg_update_combo (E2_MkdirDialogRuntime *rt)
{
	GtkWidget *entry = GTK_BIN (rt->combo)->child;
	if (rt->opt_suggest_dir || rt->opt_show_last)
	{
		gchar *text = e2_combobox_last_text (GTK_COMBO_BOX (rt->combo));
		if (text != NULL)
		{
			e2_combobox_set_active (rt->combo,
				e2_combobox_last_index (GTK_COMBO_BOX (rt->combo)));
			if (rt->opt_suggest_dir)
			{
				gchar *new_text = _e2_mkdirdlg_find_dir (text, rt->path);
				gtk_entry_set_text (GTK_ENTRY (entry), new_text);
				g_free (new_text);
			}
			g_free (text);
	//		gtk_editable_set_position (GTK_EDITABLE (entry), -1);
			gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
		}
		else
			gtk_entry_set_text (GTK_ENTRY (entry), "");
	}
	else
		gtk_entry_set_text (GTK_ENTRY (entry), "");

//		g_usleep (50000);	//maybe this fixes the sporadic combo-related crash on 'OK'?
		WAIT_FOR_EVENTS;

#ifdef RACE_CHECK
		printd (DEBUG, " mkdir update combo completed");
#endif
}
//parent is utf8
static gchar *_e2_mkdirdlg_real_parent (gchar *parent, gboolean *exists, gint *i)
{
	//find last existing parent directory
	gchar *local = F_FILENAME_TO_LOCALE (parent);
	while (e2_fs_access (local, F_OK)
			|| ! e2_fs_is_dir3 (local))
	{
		(*i)++;
		*exists = FALSE;
		if ((parent[0] == '/') && (parent[1] == '\0'))
			break;
		gchar *free = parent;
		parent = g_path_get_dirname (parent);
		F_FREE (local);
		local = F_FILENAME_TO_LOCALE (parent);
		g_free (free);
	}
	F_FREE (local);
	return parent;
}
//path is utf8
static void _e2_mkdirdlg_update_label1 (gchar *path, E2_MkdirDialogRuntime *rt)
{
	gchar *translated = e2_utils_translate_relative_path (rt->path, path);
	gchar *label = g_strconcat ("<small>", translated, "</small>", NULL);
	gtk_label_set_markup (GTK_LABEL (rt->info_label), label);
	g_free (translated);
	g_free (label);
}

static void _e2_mkdirdlg_update_label2 (gchar *path, gchar *reason, E2_MkdirDialogRuntime *rt)
{
	gchar *color;
	gchar *constant;
	if (rt->creation_possible)
	{
		constant = _("yes");
//		gtk_widget_set_sensitive (rt->button_create, TRUE);
		gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), E2_RESPONSE_CREATE, TRUE);
		gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), GTK_RESPONSE_OK, TRUE);
		color = e2_option_str_get ("color-positive");
	}
	else
	{
		constant = _("no");
//		gtk_widget_set_sensitive (rt->button_create, FALSE);
		gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), E2_RESPONSE_CREATE, FALSE);
		gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), GTK_RESPONSE_OK, FALSE);
		color = e2_option_str_get ("color-negative");
	}
	gchar *label = g_strconcat ("<span weight=\"bold\" size=\"small\" foreground=\"",
			color, "\">", constant, " </span>", reason == NULL ? NULL : "<small>(",
			reason, ")</small>", NULL);
	gtk_label_set_markup (GTK_LABEL (rt->info_label2), label);
	g_free (label);
}
/**
 * @brief update the two info labels of the info frame
 * This is done even if the info frame isn't shown, to update the 
 * rt->creation_possible state
 *
 * @param rt pointer to dialog's data struct
 *
 * @return
 */
static void _e2_mkdirdlg_update_labels (E2_MkdirDialogRuntime *rt)
{
	gboolean absolute;
	gchar *path, *parent;

	rt->creation_possible = TRUE;

	const gchar *entry_text = gtk_entry_get_text (GTK_ENTRY
		(GTK_BIN (rt->combo)->child));

	//if the entry is empty, we'll only check if the parent directory
	//is writable, thus, set path to the currently active directory
	if (*entry_text == '\0')
	{
		path = g_strdup (rt->path);
		absolute = TRUE;	//CHECKME 
		parent = g_strdup (path);
	}
	else //otherwise we have to create an absolute path from
			//the entry text and the active directory
	{
		path = (gchar *) entry_text;
		absolute = g_path_is_absolute (path);
		//if the directory entered is not absolute, concatenate it
		if (absolute)
			path = g_strdup (entry_text); //make sure this is always freeable
		else
			path = g_strconcat (rt->path, path, NULL);  //CHECKME extra separator ?

		parent = g_path_get_dirname (path);
	}

	gboolean parent_exists = TRUE;
	gint num = g_str_has_suffix (entry_text, G_DIR_SEPARATOR_S) ? 0 : 1;
	parent = _e2_mkdirdlg_real_parent (parent, &parent_exists, &num);
	gchar *local = F_FILENAME_TO_LOCALE (parent);

	_e2_mkdirdlg_update_label1 (parent, rt);
	
	gchar *reason = NULL;
	if ((e2_fs_access (local, W_OK | X_OK)) != 0)
	{
		if (absolute)
			reason = g_strdup_printf (_("cannot write to '%s': %s"),
				parent, g_strerror (errno));
		else if (parent_exists)
			reason = g_strdup_printf (_("cannot write to parent directory: %s"),
				g_strerror (errno));
		else
			reason = g_strdup_printf (_("only '%s' exists: %s"),
				parent, g_strerror (errno));
		rt->creation_possible = FALSE;
		_e2_mkdirdlg_update_label2 (path, reason, rt);
		g_free (path);
		g_free (parent);
		F_FREE (local);
		return;
	}
	F_FREE (local);
	if (entry_text[0] == '\0')
	{
		_e2_mkdirdlg_update_label2 (path, reason, rt);
		g_free (path);
		g_free (parent);
		return;
	}
	local = F_FILENAME_TO_LOCALE (path);
	if (!e2_fs_access (local, F_OK))
	{
		if  (e2_fs_is_dir3 (local))
			reason = _("the directory already exists.");
		else
			reason = _("something is in the way.");

		rt->creation_possible = FALSE;
	}
	F_FREE (local);

	if (num > 1)
		reason = g_strdup_printf ("%d %s",num,_("directories will be created"));
	_e2_mkdirdlg_update_label2 (path, reason, rt);
	g_free (parent);
	g_free (path);
}

static void _e2_mkdirdlg_update_follow_dir (E2_MkdirDialogRuntime *rt, gboolean follow)
{
	if (follow)
	{
		g_free (rt->path);
		rt->path = g_strdup (curr_pane->path);
		_e2_mkdirdlg_update_labels (rt);
		e2_hook_register (&app.pane1.hook_change_dir, _e2_mkdirdlg_change_dir_cb, rt);
		e2_hook_register (&app.pane2.hook_change_dir, _e2_mkdirdlg_change_dir_cb, rt);
		e2_hook_register (&app.hook_pane_focus_changed, _e2_mkdirdlg_change_focus_cb, rt);
	} else
	{
		e2_hook_unregister (&app.pane1.hook_change_dir, _e2_mkdirdlg_change_dir_cb, rt, TRUE);
		e2_hook_unregister (&app.pane2.hook_change_dir, _e2_mkdirdlg_change_dir_cb, rt, TRUE);
		e2_hook_unregister (&app.hook_pane_focus_changed, _e2_mkdirdlg_change_focus_cb, rt, TRUE);
	}
}
/**
 * @brief create a new directory
 * The string provided may be an absolute or relative path, the latter
 * including just a name. If relative, the last-used dir?? will be prepended
 * @param entry the entry widget for the name-entry combo
 * @param rt pointer to dialog's data struct
 * @param close TRUE to close the dialog when finished here
 *
 * @return
 */
static void _e2_mkdirdlg_create_dir (GtkWidget *entry,
	E2_MkdirDialogRuntime *rt, gboolean close)
{
	if (!rt->creation_possible)
		return;
	const gchar *dir = gtk_entry_get_text (GTK_ENTRY (entry));
	//quick exit
	if (*dir == '\0')
		return;

	gchar *path, *local;

	if (g_path_is_absolute (dir))
		path = g_strdup (dir);
	else
		path = g_strconcat (rt->path, dir, NULL);
	
	gchar *parent = g_path_get_dirname (path);
	if (g_str_has_suffix (path, G_DIR_SEPARATOR_S))
	{
		gchar *freeme = parent;
		parent = g_path_get_dirname (parent);
		g_free (freeme);
	}

	GList *all = NULL;
	//find last existing parent directory
	local = F_FILENAME_TO_LOCALE (parent);
	while (e2_fs_access (local, F_OK)
				|| !e2_fs_is_dir3 (local))
	{
		if ((parent[0] == G_DIR_SEPARATOR) && (parent[1] == '\0'))
			break;
		all = g_list_prepend (all, parent);
		parent = g_path_get_dirname (parent);
	}
	g_free (parent);
	F_FREE (local);
	
	GList *tmp;
	for (tmp = all; tmp != NULL; tmp = g_list_next (tmp))
	{
		local = F_FILENAME_TO_LOCALE ((gchar *)tmp->data);
		if (e2_fs_mkdir (local, 0777))
		{
			gchar *message = g_strdup_printf (_("Could not create directory '%s'. (%s)"), path,
				g_strerror (errno));
			e2_output_print_error (message, TRUE);
			_e2_mkdirdlg_update_labels (rt);

			e2_filelist_check_dirty (GINT_TO_POINTER(1));
			e2_list_free_with_data (&all);
			F_FREE (local);
			return;
		}
		F_FREE (local);
	}
	e2_list_free_with_data (&all);

	local = F_FILENAME_TO_LOCALE (path);
	if (!e2_fs_mkdir (local, 0777))	//succeeded
	{
		e2_filelist_check_dirty (GINT_TO_POINTER(1));
		e2_list_update_history ((gchar *) dir, &history, NULL, 30, FALSE);
		_e2_mkdirdlg_update_combo (rt);
	}
	else
	{
		gchar *message = g_strdup_printf (_("Could not create directory '%s'. (%s)"), path,
			g_strerror (errno));
		e2_output_print_error (message, TRUE);
	}
	g_free (path);
	F_FREE (local);
	if (!close)
		_e2_mkdirdlg_update_labels (rt);
}

  /*********************/
 /***** callbacks *****/
/*********************/

static gboolean _e2_mkdirdlg_changed_cb_idle (E2_MkdirDialogRuntime *rt)
{
//	gdk_threads_enter ();	//protect idle fn against thread glitches
	_e2_mkdirdlg_update_dialog_size (rt);
//	gdk_threads_leave ();
	rt->idle_id = 0;
	return FALSE;
}

static gboolean _e2_mkdirdlg_changed_cb (GtkWidget *entry, E2_MkdirDialogRuntime *rt)
{
	_e2_mkdirdlg_update_labels (rt);
	if (rt->idle_id != 0)
		g_source_remove (rt->idle_id);
	rt->idle_id = g_idle_add ((GSourceFunc) _e2_mkdirdlg_changed_cb_idle, rt);
	return FALSE;
}
/**
 * @brief dialog response callback
 *
 * @param dialog the dialog where the response was initiated
 * @param response the response assigned to the activated button
 * @param rt pointer to dialog's data struct
 * @return
 */
static void _e2_mkdirdlg_response_cb (GtkWidget *dialog, gint response,
	E2_MkdirDialogRuntime *rt)
{
	printd (DEBUG, "response_cb (dialog:_,response:%d,rt:_)", response);
	GtkWidget *entry = GTK_BIN (rt->combo)->child;
	switch (response)
	{
		case E2_RESPONSE_CREATE:
			e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
			_e2_mkdirdlg_create_dir (entry, rt, FALSE);
			gtk_widget_grab_focus (entry);
			break;
		case GTK_RESPONSE_OK:
		{
			e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
			_e2_mkdirdlg_create_dir (entry, rt, TRUE);
			_e2_mkdirdlg_dialog_destroy (rt);
			break;
		}
		default:
			_e2_mkdirdlg_dialog_destroy (rt);
			break;
	}
}
/**
 * @brief combobox activate callback
 * This simply triggers an 'ok' type response
 * @param entry UNUSED the entry widget for the name-entry combo
 * @param rt pointer to dialog's data struct
 * @return
 */
static void _e2_mkdirdlg_activate_cb (GtkWidget *entry, E2_MkdirDialogRuntime *rt)
{
	_e2_mkdirdlg_response_cb (rt->dialog, GTK_RESPONSE_OK, rt);
}

//assumes path is locale-encoded
static gboolean _e2_mkdirdlg_change_dir_cb (gchar *path, E2_MkdirDialogRuntime *rt)
{
	g_free (rt->path);
	rt->path = g_strdup (path);
	_e2_mkdirdlg_update_labels (rt);
	_e2_mkdirdlg_update_dialog_size (rt);
	return TRUE;
}

static gboolean _e2_mkdirdlg_change_focus_cb (E2_PaneRuntime *pane_rt, E2_MkdirDialogRuntime *rt)
{
	g_free (rt->path);
	rt->path = g_strdup (pane_rt->path);
	_e2_mkdirdlg_update_labels (rt);
	_e2_mkdirdlg_update_dialog_size (rt);
	return TRUE;
}

static void _e2_mkdirdlg_toggled_cb1 (GtkWidget *menu_item, E2_MkdirDialogRuntime *rt)
{
	rt->opt_show_last = gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item));
	_e2_mkdirdlg_update_combo (rt);
}

static void _e2_mkdirdlg_toggled_cb2 (GtkWidget *menu_item, E2_MkdirDialogRuntime *rt)
{
	rt->opt_suggest_dir = gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item));
	_e2_mkdirdlg_update_combo (rt);
}

static void _e2_mkdirdlg_toggled_cb3 (GtkWidget *menu_item, E2_MkdirDialogRuntime *rt)
{
	_e2_mkdirdlg_update_follow_dir (rt, gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item)));
}

static void _e2_mkdirdlg_toggled_expander_cb (GtkWidget *expander,
	GParamSpec *spec, E2_MkdirDialogRuntime *rt)
{
	_e2_mkdirdlg_update_dialog_size (rt);
}

static void _e2_mkdirdlg_show_cb (GtkWidget *dialog, E2_MkdirDialogRuntime *rt)
{
	_e2_mkdirdlg_update_combo (rt);
	gtk_expander_set_expanded (GTK_EXPANDER (rt->info_expander),
		e2_option_bool_get ("dialog-mkdir-show-info"));
}
/**
 * @brief create and show the mkdir dialog
 * @return
 */
static void _e2_mkdir_dialog_create (void)
{
	//init runtime object
	E2_MkdirDialogRuntime *rt = g_malloc (sizeof (E2_MkdirDialogRuntime));
	rt->history = e2_list_copy_with_data (history);
	rt->path = g_strdup (curr_pane->path);
	rt->creation_possible = TRUE;
	rt->idle_id = 0;
	
	//create dialog
	rt->dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
		_("What is the new directory's name?"), _("create directory"),
		_e2_mkdirdlg_response_cb, rt);

	GtkWidget *vbox = e2_dialog_add_sw (rt->dialog);
	rt->scrolled = vbox->parent->parent;
	
	//add combo
	rt->combo = e2_combobox_add (vbox,
		FALSE, E2_PADDING, _e2_mkdirdlg_activate_cb, rt,
		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
	e2_combobox_append_history (rt->combo, history);

	//add info box with expander
	rt->info_box = e2_widget_get_box (TRUE, TRUE, TRUE);
	rt->info_expander = e2_option_bool_add_expander_widget (rt->dialog,
		vbox, _("info"), e2_option_get ("dialog-mkdir-show-info"), rt->info_box);
	gtk_widget_show (rt->info_expander);
	g_signal_connect (G_OBJECT (rt->info_expander), "notify::expanded",
		G_CALLBACK (_e2_mkdirdlg_toggled_expander_cb), rt);

	GtkSizeGroup *group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	GtkWidget *hbox = e2_widget_add_box (rt->info_box, FALSE, 0, FALSE, FALSE, 0);
	GtkWidget *label = _e2_widget_add_label (hbox,
		_("<small>parent directory:</small>"), 0.0, 0.0, FALSE, E2_PADDING_SMALL);
	gtk_size_group_add_widget (group, label);

	rt->info_label = e2_widget_add_label (hbox, "", 0.0, TRUE, E2_PADDING_SMALL);
	hbox = e2_widget_add_box (rt->info_box, FALSE, 0, FALSE, FALSE, 0);
	label = _e2_widget_add_label (hbox,
		_("<small>creation possible:</small>"), 0.0, 0.0, FALSE, E2_PADDING_SMALL);
	gtk_size_group_add_widget (group, label);
	rt->info_label2 = _e2_widget_add_label (hbox, "", 0.0, 0.0, TRUE, E2_PADDING_SMALL);

	E2_OptionSet *connected = e2_option_get ("dialog-mkdir-connected");
	E2_OptionSet *follow_pane = e2_option_get ("dialog-mkdir-follow-pane");
	E2_OptionSet *show_last = e2_option_get ("dialog-mkdir-show-last");
	E2_OptionSet *suggest_dir = e2_option_get ("dialog-mkdir-suggest-directory");

	rt->menu = e2_menu_create_options_menu (rt->dialog, NULL,
		connected, e2_menu_control_cb, rt->dialog,
		follow_pane, _e2_mkdirdlg_toggled_cb3, rt,
		show_last, _e2_mkdirdlg_toggled_cb1, rt,
		suggest_dir, _e2_mkdirdlg_toggled_cb2, rt,
		NULL);

	g_signal_connect (G_OBJECT (GTK_BIN (rt->combo)->child), "changed",
		G_CALLBACK (_e2_mkdirdlg_changed_cb), rt);
	g_signal_connect (G_OBJECT (rt->dialog), "show", G_CALLBACK (_e2_mkdirdlg_show_cb), rt);
	_e2_mkdirdlg_update_follow_dir (rt, e2_option_bool_get ("dialog-mkdir-follow-pane"));

	e2_option_connect (rt->dialog, e2_option_bool_get ("dialog-mkdir-connected"));
	e2_dialog_attach_menu (rt->dialog, rt->menu);

	rt->opt_show_last = e2_option_bool_get ("dialog-mkdir-show-last");
	rt->opt_suggest_dir = e2_option_bool_get ("dialog-mkdir-suggest-directory");
	
	E2_BUTTON_CANCEL.showflags |= E2_BTN_DEFAULT;
	E2_BUTTON_OK.showflags &= !E2_BTN_DEFAULT;
	e2_dialog_show (rt->dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL,
		&E2_BUTTON_CANCEL, &E2_BUTTON_CREATE, &E2_BUTTON_OK, NULL);
}

  /******************/
 /***** public *****/
/******************/

void e2_mkdir_dialog_actions_register ()
{
	gchar *action_name = g_strconcat(_A(3),".",_A(52),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_mkdir_dialog_create, NULL, FALSE);
}

void e2_mkdir_dialog_options_register ()
{
	e2_cache_list_register ("mkdir-history", &history);

	gchar* group_name = g_strconcat(_C(9),":",_C(22),NULL);   //_("dialogs:mkdir"
	e2_option_bool_register ("dialog-mkdir-connected", group_name, _("dialog connected"),
		_("This causes replication of option changes to other 'connected' mkdir dialogs. Otherwise such changes will be temporary to the current dialog"),
		NULL, TRUE,
		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP);
	e2_option_bool_register ("dialog-mkdir-follow-pane",
		group_name, _("follow pane directory"),
		_("This makes the parent directory for new directories the same as the one in the active pane, even if the latter changes"),
		NULL, TRUE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register ("dialog-mkdir-show-info",
		group_name, _("show info frame"),
		_("This causes extra information to be shown in the make-directory dialog"), NULL, TRUE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register ("dialog-mkdir-suggest-directory",
		group_name, _("suggest new directory name"),
		_("This presents a suggested name for each new directory, based on the last-created directory with an increasing number appended"), 
		NULL, TRUE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register ("dialog-mkdir-show-last",
		group_name, _("show last directory name"),
		_("This casues the name of the last-created directory to be shown in the entry field, after opening the dialog or when creating another directory"),
			"!dialog-mkdir-suggest-directory", FALSE,
		E2_OPTION_FLAG_ADVANCED);
}
