/* $Id: e2_permissions_dialog.c 1549 2006-01-29 10:52:49Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <pwd.h>
#include <grp.h>
#include "e2_dialog.h"
#include "e2_permissions_dialog.h"
#include "e2_ownership_dialog.h"

enum { SETUID, SETGID, STICKY, USR_READ, USR_WRITE, USR_EXEC, GRP_READ,
       GRP_WRITE, GRP_EXEC, OTH_READ, OTH_WRITE, OTH_EXEC };

//static GtkWidget *permissions_dialog;
static mode_t mask[12] = {S_ISUID, S_ISGID, S_ISVTX, S_IRUSR, S_IWUSR, S_IXUSR,
                          S_IRGRP, S_IWGRP, S_IXGRP, S_IROTH, S_IWOTH, S_IXOTH};
//strings used for command string, not translated
static gchar *add_strings[12] = 
		{"u+s", "g+s", "o+t", "u+r", "u+w", "u+x", "g+r", "g+w", "g+x", "o+r", "o+w", "o+x"};
static gchar *remove_strings[12] = 
		{"u-s", "g-s", "o-t", "u-r", "u-w", "u-x", "g-r", "g-w", "g-x", "o-r", "o-w", "o-x"};
static GtkWidget *chmod_buttons[12];
static GtkWidget *set_perms_button;
static GtkWidget *add_perms_button;
static GtkWidget *remove_perms_button;
static GtkWidget *recurse_button;
static GtkWidget *recurse_dirs_button;
static GtkWidget *recurse_other_button;
static E2_RecurseType *recurse;
static gboolean *permission;	
static GString **mode;

static GString *_e2_permissions_dialog_get_mode_string ()
{
  gint i;
  GString *mode_string = g_string_new ("");
  if (GTK_TOGGLE_BUTTON (set_perms_button)->active)
  {
    mode_t mode = 0;
    for (i = 0; i < 12; i++)
    {
      if (GTK_TOGGLE_BUTTON (chmod_buttons[i])->active)
        mode |= mask[i];
    }
    g_string_printf (mode_string, "%o", (guint) mode);
  }
  else if (GTK_TOGGLE_BUTTON (add_perms_button)->active)
  {
    for (i = 0; i < 12; i++)
    {
      if (GTK_TOGGLE_BUTTON (chmod_buttons[i])->active)
      {
        g_string_append (mode_string, add_strings[i]);
        g_string_append_c (mode_string, ',');
      }
    }
    i = mode_string->len;
    if (i > 0)
      g_string_truncate (mode_string, i-1); // get rid of trailing comma
  }
  else if (GTK_TOGGLE_BUTTON (remove_perms_button)->active)
  {
    for (i = 0; i < 12; i++)
    {
      if (GTK_TOGGLE_BUTTON (chmod_buttons[i])->active)
      {
        g_string_append (mode_string, remove_strings[i]);
        g_string_append_c (mode_string, ',');
      }
    }
    i = mode_string->len;
    if (i > 0)
      g_string_truncate (mode_string, i-1); // get rid of trailing comma
  }
  else
  {
	printd (DEBUG, "Strange error in permissions dialog.");
    g_string_free (mode_string, TRUE);
    return NULL;
  }
  return mode_string;
}

static void _e2_permissions_dialog_ok_cb (GtkWidget *widget)
{
  if (*permission)  //test prevents crash
  {
	*mode = _e2_permissions_dialog_get_mode_string ();
	if (recurse_button != NULL)
	{  //the recurse button has been added to the dialog
	  *recurse = E2_RECURSE_NONE;
	  if (GTK_TOGGLE_BUTTON (recurse_button)->active)
	  {
		  if (GTK_TOGGLE_BUTTON (recurse_dirs_button)->active)
			*recurse = E2_RECURSE_DIRS;
		  if (GTK_TOGGLE_BUTTON (recurse_other_button)->active)
			*recurse |= E2_RECURSE_OTHER;
		  if (*recurse == (E2_RECURSE_DIRS | E2_RECURSE_OTHER))
			*recurse = E2_RECURSE_ALL;
	  }
	}
 }
  return;
}

/*static void stop_cb (GtkWidget *widget)
{
	return;
}*/

static void _e2_permissions_dialog_add_chmod_button (
				GtkWidget *table,   //was *box,
				gint i,
				gchar *label,
				gint state, 
				gint left, gint right, gint top, gint bottom)  //new
{
  if (label == NULL)
  {    //align the button reasonably
	GtkWidget *aligner =  gtk_alignment_new (0.1 , 0.5, 0, 0 );
	GtkWidget *check_button = gtk_check_button_new ();
	gtk_container_add (GTK_CONTAINER(aligner), check_button);
	gtk_table_attach ( GTK_TABLE (table), aligner, 
		left, right, top, bottom,
		GTK_FILL,  //GtkAttachOptions xoptions
		GTK_FILL, //GtkAttachOptions yoptions
		0, //xpadding
		0 //ypadding
		);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button), state);
	gtk_widget_show (check_button);
	chmod_buttons[i] = check_button;
  }
  else  //there is a label
	chmod_buttons[i] = e2_button_add_toggle_to_table (table, 
		    label, state, NULL, NULL, left, right, top, bottom);
  //prevent changes if relevant
  if (!*permission)
		gtk_widget_set_sensitive (chmod_buttons[i], FALSE);
}

static void _e2_permissions_dialog_clear_chmod_buttons_cb (GtkWidget *widget)
{
  gint i;
  for (i = 0; i < 12; i++)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chmod_buttons[i]), FALSE);
}

static void _e2_permissions_dialog_reset_chmod_buttons_cb 
	(GtkWidget *widget, gpointer mode)
{
  mode_t st_mode = (mode_t) mode;
  gboolean flag;
  gint i;
  for (i = 0; i < 12; i++)
  {
	flag = st_mode & mask[i];
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chmod_buttons[i]), flag);
  }
}

static void _e2_permissions_dialog_toggle_recurse_button_cb (GtkWidget *widget)
{
	if (!*permission)
		return;	
	gboolean flag = (GTK_TOGGLE_BUTTON (widget)->active);
	gtk_widget_set_sensitive (recurse_dirs_button, flag);
	gtk_widget_set_sensitive (recurse_other_button, flag);
}

GtkWidget *e2_permissions_dialog_setup (
					E2_SelectedItemInfo *info,
					E2_RecurseType *recurse_ret,
					gboolean *permission_ret,
					GString    **mode_ret,
					gboolean multi )
{
  GtkWidget *permissions_dialog;  // non-static = ok when data retrieved before dialog closed
  GtkWidget *dialog_vbox, *sub_vbox;
  GtkWidget *hbox;
  GtkWidget *frame;
  GtkWidget *action_area;
  GtkWidget *table;

  GString *label_text = g_string_sized_new (NAME_MAX+20);
  struct stat statbuf;
  struct passwd *pw_buf;
  struct group *grp_buf;

  // Set up pointers to the static values
  recurse = recurse_ret;
  permission = permission_ret;
  mode = mode_ret;

//  gint myuid = getuid ();

//  gchar *itempath = g_strconcat (curr_view->dir, info->filename, NULL);
//  e2_fs_e2_fs_lstat (itempath, &statbuf);
//  *permission = e2_utils_check_write_permission (-1, itempath);
//  g_free (itempath);
  //path is to current dir
  gchar *local = F_FILENAME_TO_LOCALE (info->filename);
  e2_fs_lstat (local, &statbuf);
  F_FREE (local);
 *permission = e2_fs_check_write_permission (-1, info->filename);

  permissions_dialog = gtk_dialog_new ();
  E2_DIALOG_TITLE (GTK_WINDOW (permissions_dialog), _("permissions"));
  gtk_dialog_set_has_separator (GTK_DIALOG (permissions_dialog), FALSE);  //all info in frames, no need
  dialog_vbox = GTK_DIALOG (permissions_dialog)->vbox;
  action_area = GTK_DIALOG (permissions_dialog)->action_area;
  gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);

//  table = e2_widget_add_table (dialog_vbox, 3, 3, TRUE, TRUE, 0);  //3 rows, 3 cols, homogen, fill, no pad
//  gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
  gboolean thisis_dir = e2_fs_is_dir2 (info);
  gchar *label = (thisis_dir) ? _("Directory name") : _("Filename") ;
  g_string_printf (label_text, "%s: <b>%s</b>", label, info->filename);

//  e2_widget_add_label_to_table (table, label_text->str, 0, 
//  		1, 2, 0, 1);  //     gint left col, gint right col, gint top row, gint bottom row)
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, E2_PADDING); //top, bottom padding 
  e2_widget_add_label (hbox, label_text->str, 0, TRUE, E2_PADDING); //L, R padding
  gtk_widget_show (hbox);

  label = _("User");
  if ((pw_buf = getpwuid (statbuf.st_uid)) != NULL)
    g_string_printf (label_text, "%s: %s", label, pw_buf->pw_name);
  else
    g_string_printf (label_text, "%s: %d", label, (guint) statbuf.st_uid);
//  e2_widget_add_label_to_table (table, label_text->str, 0, 1, 2, 1, 2);
  hbox = gtk_hbox_new (FALSE, 0);
  e2_widget_add_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
  gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);

  label = _("Group");
  if ((grp_buf = getgrgid (statbuf.st_gid)) != NULL)
    g_string_printf (label_text, "%s: %s", label, grp_buf->gr_name);
  else
    g_string_printf (label_text, "%s: %d", label, (guint) statbuf.st_gid);
//  e2_widget_add_label_to_table (table, label_text->str, 0, 1, 2, 2, 3);
  hbox = gtk_hbox_new (FALSE, 0);
  e2_widget_add_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
  gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);

  frame = gtk_frame_new (_("Permissions"));
  gtk_box_pack_start (GTK_BOX (dialog_vbox), frame, TRUE, TRUE, E2_PADDING);
  gtk_widget_show (frame);

  sub_vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
  gtk_widget_show (sub_vbox);

  table = e2_widget_add_table (sub_vbox, 4, 5, TRUE, TRUE, 0);  //4 rows, 5 cols, homogen, fill, no pad
  e2_widget_add_label_to_table (table, _("Read"), 0,			1,2,0,1);  //gint left, gint right, gint top, gint bottom)
  e2_widget_add_label_to_table (table, _("Write"), 0,			2,3,0,1);
  e2_widget_add_label_to_table (table, _("Exec"), 0.05,	3,4,0,1);
  e2_widget_add_label_to_table (table, _("Special"), 0.2,	4,5,0,1);

  e2_widget_add_label_to_table (table, _("User"), 0.1,		0,1,1,2);
  _e2_permissions_dialog_add_chmod_button (table, USR_READ, NULL, statbuf.st_mode & S_IRUSR, 	1,2,1,2);
  _e2_permissions_dialog_add_chmod_button (table, USR_WRITE, NULL, statbuf.st_mode & S_IWUSR,	2,3,1,2);
  _e2_permissions_dialog_add_chmod_button (table, USR_EXEC, NULL, statbuf.st_mode & S_IXUSR,	3,4,1,2);
  _e2_permissions_dialog_add_chmod_button (table, SETUID, _("Set UID"), statbuf.st_mode & S_ISUID,	4,5,1,2);

  e2_widget_add_label_to_table (table, _("Group"), 0.1,		0,1,2,3);
  _e2_permissions_dialog_add_chmod_button (table, GRP_READ, NULL, statbuf.st_mode & S_IRGRP,	1,2,2,3);
  _e2_permissions_dialog_add_chmod_button (table, GRP_WRITE, NULL, statbuf.st_mode & S_IWGRP, 2,3,2,3);
  _e2_permissions_dialog_add_chmod_button (table, GRP_EXEC, NULL, statbuf.st_mode & S_IXGRP,	3,4,2,3);
  _e2_permissions_dialog_add_chmod_button (table, SETGID, _("Set GID"), statbuf.st_mode & S_ISGID,	4,5,2,3);

  e2_widget_add_label_to_table (table, _("Other"), 0.1,		0,1,3,4);
  _e2_permissions_dialog_add_chmod_button (table, OTH_READ, NULL, statbuf.st_mode & S_IROTH,   1,2,3,4);
  _e2_permissions_dialog_add_chmod_button (table, OTH_WRITE, NULL, statbuf.st_mode & S_IWOTH, 2,3,3,4);
  _e2_permissions_dialog_add_chmod_button (table, OTH_EXEC, NULL, statbuf.st_mode & S_IXOTH,    3,4,3,4);
  _e2_permissions_dialog_add_chmod_button (table, STICKY, _("Sticky"), statbuf.st_mode & S_ISVTX,    4,5,3,4); 

  if (*permission)
  {
	frame = gtk_frame_new (_("Action:"));
	gtk_box_pack_start (GTK_BOX (dialog_vbox), frame, TRUE, TRUE, E2_PADDING);
	gtk_widget_show (frame);

	sub_vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
	gtk_widget_show (sub_vbox);

 	table = e2_widget_add_table (sub_vbox, 1, 5, TRUE, TRUE, 0);  //1 rows, 5 cols, homogen, fill, no pad
    set_perms_button = e2_button_add_radio_to_table (table, _("_Set"),
		NULL, TRUE, _e2_permissions_dialog_reset_chmod_buttons_cb,
		(gpointer) statbuf.st_mode,    //info, 
		1, 2, 0, 1 ); //gint left, gint right, gint top, gint bottom
	add_perms_button = e2_button_add_radio_to_table (table, _("_Add"),
        gtk_radio_button_get_group (GTK_RADIO_BUTTON (set_perms_button)),
		FALSE, _e2_permissions_dialog_clear_chmod_buttons_cb, NULL, 
	  	2, 3, 0, 1);
	remove_perms_button = e2_button_add_radio_to_table (table, _("_Remove"),
		gtk_radio_button_get_group (GTK_RADIO_BUTTON (add_perms_button)),
		FALSE, _e2_permissions_dialog_clear_chmod_buttons_cb, NULL,
		3, 4, 0, 1);

	 if (thisis_dir)  //no longert do this for 'multi'
	 {
		table = e2_widget_add_table (sub_vbox, 1, 5, TRUE, TRUE, 0);  //1 row, 5 cols, homogen, fill, no pad
		recurse_button = e2_button_add_toggle_to_table (table, _("R_ecurse:"),
			FALSE, _e2_permissions_dialog_toggle_recurse_button_cb, NULL,  
			1,2,0,1 );
//		recurse_all_button = e2_button_add_radio_to_table (table, _("a_ll"),
//			NULL, TRUE, recurse_type_button_cb, NULL, 
//			2,3,0,1 ); //gint left, gint right, gint top, gint bottom
 		recurse_dirs_button = e2_button_add_toggle_to_table (table, _("_directories"),
		   TRUE, NULL, NULL, 2,3,0,1) ;
		gtk_widget_set_sensitive (recurse_dirs_button, FALSE);
//		recurse_dirs_button = e2_button_add_radio_to_table (table, _("_directories"),
//			gtk_radio_button_get_group (GTK_RADIO_BUTTON (recurse_all_button)),
//			FALSE, recurse_type_button_cb, NULL,
//			3,4,0,1);
  		recurse_other_button = e2_button_add_toggle_to_table (table, _("_others"),
		   TRUE, NULL, NULL, 3,4,0,1) ;
		gtk_widget_set_sensitive (recurse_other_button, FALSE);
	 }
  }
  else //no right to change object permissions
  {
	e2_ownership_dialog_warn (dialog_vbox);
  }
  g_string_free (label_text, TRUE);

// add buttons in the order that they will appear
  GtkWidget *btn;
  if (multi)
	{
		e2_dialog_add_defined_button (permissions_dialog, &E2_BUTTON_NOTOALL);
		btn = e2_dialog_add_button_custom (permissions_dialog, FALSE,
			&E2_BUTTON_APPLYTOALL, NULL, _e2_permissions_dialog_ok_cb, NULL);
		if (!*permission)
			gtk_widget_set_sensitive (btn, FALSE);
	}
	if (*permission || multi)
	{
		btn = e2_dialog_add_defined_button (permissions_dialog, &E2_BUTTON_CANCEL);
		if (!*permission)
			gtk_widget_set_sensitive (btn, FALSE);
	}
	e2_dialog_add_button_custom (permissions_dialog, TRUE, &E2_BUTTON_OK,
		NULL, _e2_permissions_dialog_ok_cb, NULL);

	e2_dialog_setup (permissions_dialog, app.main_window);
	return permissions_dialog;
}
