/* $Id: e2_view_dialog.c 1856 2006-04-24 12:51:28Z tpgww $
 * 
 * Copyright (C) 2004-2006 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_dialog.h"
#include "e2_view_dialog.h"
#include "e2_textiter.h"

gboolean show_last;
gboolean case_sensitive;
gboolean search_backward;
gboolean search_wrap;
GList *find_history = NULL;
GList *view_history = NULL;
//keycodes corresponding to mnemonics for translated labels
guint find_keycode;
guint hide_keycode;
//we need this one because of widget-sharing with the edit dialog
//value set to 0 for a view dialog
extern guint replace_keycode;

static void _e2_view_dialog_response_cb
	(GtkWidget *dialog, gint response, E2_ViewDialogRuntime *rt);

  /***************************/
 /***** search functions *****/
/***************************/

void e2_view_dialog_update_combo (GtkWidget *combo)
{
	GtkWidget *entry = GTK_BIN (combo)->child;
	if (show_last)
	{
		gchar *text = e2_combobox_last_text (GTK_COMBO_BOX (combo));
		if (text != NULL)
		{
			e2_combobox_set_active (combo, e2_combobox_last_index (GTK_COMBO_BOX (combo)));
//			gtk_entry_set_text (GTK_ENTRY (entry), text); redundant when combo is set
			g_free (text);
			gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
		}
		else
			gtk_entry_set_text (GTK_ENTRY (entry), "");
	}
	else
		gtk_entry_set_text (GTK_ENTRY (entry), "");
}

static void _e2_view_dialog_attach_search_iters (GtkTextBuffer *buffer,
	GtkTextIter *start, GtkTextIter *end)
{
	GtkTextMark *mark = gtk_text_buffer_get_mark (buffer, "e2-search-start");
	if (mark == NULL)
		gtk_text_buffer_create_mark (buffer, "e2-search-start", start, FALSE);
	else
		gtk_text_buffer_move_mark (buffer, mark, start);
	mark = gtk_text_buffer_get_mark (buffer, "e2-search-end");
	if (mark == NULL)
		gtk_text_buffer_create_mark (buffer, "e2-search-end", end, FALSE);
	else
		gtk_text_buffer_move_mark (buffer, mark, end);
}

static void _e2_view_dialog_unattach_search_iters (GtkTextBuffer *buffer)
{
	GtkTextMark *mark = gtk_text_buffer_get_mark (buffer, "e2-search-start");
	if (mark != NULL)
		gtk_text_buffer_delete_mark (buffer, mark);
	mark = gtk_text_buffer_get_mark (buffer, "e2-search-end");
	if (mark != NULL)
		gtk_text_buffer_delete_mark (buffer, mark);
}

static gboolean _e2_view_dialog_dosearch (GtkTextIter *search_iter,
	GtkTextIter *start, GtkTextIter *end, const gchar *find,
	E2_ViewDialogRuntime *rt)
{
	GtkSourceSearchFlags flags = GTK_SOURCE_SEARCH_TEXT_ONLY;
	if (!case_sensitive)
		flags |= GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
	if (search_backward)
		return e2_iter_backward_search (search_iter, find,
			flags, start, end, NULL);
	else
		return e2_iter_forward_search (search_iter, find,
			flags, start, end, NULL);
}

gboolean e2_view_dialog_search (gboolean first, gboolean incremental,
	E2_ViewDialogRuntime *rt)
{
	// get the find string
	GtkWidget *entry = GTK_BIN (rt->combo)->child;
	const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
	if (*find == '\0')
		return FALSE;
	if (!incremental)
	{
		e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
		e2_list_update_history (find, &find_history, NULL, 30, FALSE);
		e2_view_dialog_update_combo (rt->combo);
	}
	
	//check if the TextView's buffer has changed and get new TextIters
	GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (rt->textview));
/*	if (buffer != rt->textbuffer)
	{
		rt->textbuffer = buffer;
		unattach_iters (buffer);
		search_iter = NULL;
	}
*/
	GtkTextMark *mark = NULL;
	if (first)
	{	//searching starts from cursor, so move that
		_e2_view_dialog_unattach_search_iters (buffer);	//(re)start searching
		GtkTextIter iter;
		if (search_backward)
			gtk_text_buffer_get_end_iter (buffer, &iter);
		else
			gtk_text_buffer_get_start_iter (buffer, &iter);
		gtk_text_buffer_place_cursor (buffer, &iter);
	}
	else	//find next
	{
		gboolean flag = (incremental) ? TRUE : search_backward;
		if (flag)
			mark = gtk_text_buffer_get_mark (buffer, "e2-search-start");
		else
			mark = gtk_text_buffer_get_mark (buffer, "e2-search-end");
	}

	GtkTextIter *search_iter = NULL;
	GtkTextIter iter;
	if (mark != NULL)
	{
		gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
		search_iter = &iter;
	}
	if (search_iter == NULL)
	{
		//search from cursor
		mark = gtk_text_buffer_get_mark (buffer, "insert");
		gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
		search_iter = &iter;
/* MAYBE search from top/bottom of window if cursor is off-window ?
		GdkRectangle visible;
		gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (rt->textview),
			&visible);
		if (rt->opt_search_backward)
			//search backward from end of screen
			gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (rt->textview),
				&iter, visible.width, visible.height);
		else
			//search forward from top of screen
			gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (rt->textview),
				&iter, visible.x, visible.y); */
	}

	// search for the string
	GtkTextIter start, end;
	gboolean found = _e2_view_dialog_dosearch (search_iter, &start, &end, find, rt);

	if (!found && search_wrap)
	{
		_e2_view_dialog_unattach_search_iters (buffer);
		GtkTextIter iter;
		if (search_backward)
			gtk_text_buffer_get_end_iter (buffer, &iter);
		else
			gtk_text_buffer_get_start_iter (buffer, &iter);
		found = _e2_view_dialog_dosearch (&iter, &start, &end, find, rt);
	}
	if (found)
	{
		gtk_widget_hide (rt->info_label);
		gtk_text_buffer_select_range (buffer, &start, &end);
		_e2_view_dialog_attach_search_iters (buffer, &start, &end);
		gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (rt->textview), &start,
			0.2, TRUE, 0.2, 0.5);
		return TRUE;
	}
	gtk_widget_show (rt->info_label);
	return FALSE;
}

  /*****************/
 /***** utils *****/
/*****************/

/**
 * @brief read text file to be viewed
 * This is used by both view and edit dialogs
 * Character encoding is performed if necessary and possible.
 * @a rt ->filepath has the name of the file to be read. It will be
 * replaced by a newly-allocated string, to be freed elsewhere
 * @param rt dialog runtime data struct
 *
 * @return TRUE if the read and conversion was successful
 **/
gboolean e2_view_dialog_read_text (E2_ViewDialogRuntime *rt)
{
	gchar *file;
	if (!g_path_is_absolute (rt->filepath))
		file = g_build_filename (curr_view->dir, rt->filepath, NULL);
	else
		file = g_strdup (rt->filepath);
	gchar *local = F_FILENAME_TO_LOCALE (file);
	if (e2_fs_access (local, F_OK))
	{
		gchar *msg = g_strdup_printf (_("'%s' does not exist"), file);
		e2_output_print_error (msg, TRUE);
		g_free (file);
		F_FREE (local);
		return FALSE;
	}
	if (e2_fs_is_dir3 (local))
	{
		gchar *msg = g_strdup_printf (_("'%s' is a directory"), file);
		e2_output_print_error (msg, TRUE);
		g_free (file);
		F_FREE (local);
		return FALSE;
	}
	F_FREE (local);
	gchar *cont, *converter = NULL;
	if (e2_option_bool_get ("use-external-encoder"))
	{ 
		converter = e2_option_str_get ("external-encoder");
		if (*converter != '\0')
		{
			gchar *command = g_strconcat (converter, " ", file, NULL);
			if (e2_fs_get_command_output (command, &cont))
				rt->charset = "UNKNOWN";	//no translation
			else
			{
				gchar *msg = g_strdup_printf 
					(_("Encoding conversion command '%s' failed"), converter);
				e2_output_print_error (msg, TRUE);
				converter = NULL;	//set flag for internal conversion fallback
			}
			g_free (command);
		}
		else
			converter = NULL;
	}
	if (converter == NULL	//not externally converted
		&& !e2_fs_get_file_contents (file, &cont))
	{
		printd (WARN, "could not open file '%s' for reading", file);
		gchar *msg = g_strdup_printf (_("Error reading file %s"), file);
		e2_output_print_error (msg, TRUE);
		g_free (file);
		return FALSE;
	}

	//fix CR's in the text if necessary
	rt->linebreak = e2_utils_LF_line_ends (cont);
	//fix character encoding if necessary and possible
	if (converter == NULL)	//not externally converted
	{
		if (*cont != '\0')
		{
			gchar *format = _("Conversion from %s encoding failed: \"%s\"");
			gchar *msg, *utf;
			GError *utf_error;
			gboolean usable = e2_utf8_detect_charset (cont, &rt->charset);
			if (!usable)
			{	//convert using the idendentified encoding if we can
				//but no point in 'non-conversion' ...
				if (strstr (rt->charset, "UTF-8") == NULL)
				{
					utf_error = NULL;
					utf = g_convert (cont, -1, "UTF-8", rt->charset, NULL, NULL, &utf_error);
					if (utf != NULL)
					{
						usable = TRUE;
						g_free (cont);
						cont = utf;
					}
					else
					{
						if (!g_str_equal (rt->charset, "CP1252"))	//that's generally a default when we can't find the real value
						{
							msg = g_strdup_printf (format, rt->charset, utf_error->message);
							e2_output_print_error (msg, TRUE);
						}
						g_error_free (utf_error);
					}
				}
			}	
			if (!usable)
			{	//try default coding as a fallback
				const gchar *defset;
				e2_utils_get_charset (&defset);
				//again, no point in 'non-conversion'
				if (strstr (defset, "UTF-8") != NULL)
					defset = e2_cl_options.fallback_encoding;
				if (strstr (defset, "UTF-8") != NULL)
					defset = "ISO-8859-1";
				printd (DEBUG, "Conversion from %s encoding to be attempted", defset);
				utf_error = NULL;
				utf = g_convert (cont, -1, "UTF-8", defset, NULL, NULL, &utf_error);
				if (utf == NULL)
				{
					msg = g_strdup_printf (format, defset, utf_error->message);
					e2_output_print_error (msg, TRUE);
					g_error_free (utf_error);
					g_free (cont);
					return FALSE;
				}
				else if (!g_utf8_validate (utf, -1, NULL))
				{
					g_free (utf);
					g_free (cont);
					return FALSE;
				}
				rt->charset = "UNKNOWN";	//prevent attempts at saving to the unusable charset 
				g_free (cont);
				cont = utf;
			}
		}
		else //assume default encoding for empty files
			e2_utils_get_charset (&rt->charset);
	}
	rt->filepath = file;	//we're ok, so remember the real path

	//for some reason, the buffer sometimes takes a while to be created 
	while (!GTK_IS_TEXT_BUFFER (rt->textbuffer))
		rt->textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (rt->textview));

	gtk_text_buffer_set_text (rt->textbuffer, cont, -1);
	g_free (cont);

	//put cursor at start of buffer, for searching etc
	GtkTextIter start;
	gtk_text_buffer_get_start_iter (rt->textbuffer, &start);
	gtk_text_buffer_place_cursor (rt->textbuffer, &start);

	return TRUE;
}
/**
 * @brief set popup menu position
 *
 * This function is supplied when calling gtk_menu_popup(), to position
 * the displayed menu.
 * set @a push_in to TRUE for menu completely inside the screen,
 * FALSE for menu clamped to screen size
 *
 * @param menu the GtkMenu to be positioned
 * @param x	place to store gint representing the menu left
 * @param y  place to store gint representing the menu top
 * @param push_in place to store pushin flag
 * @param textview output pane widget in focus when the menu key was pressed
 *
 * @return
 **/
void e2_view_dialog_set_menu_position (GtkWidget *menu,
	gint *x, gint *y, gboolean *push_in, E2_ViewDialogRuntime *rt)
{
	gint left, top;
	gtk_window_get_position (GTK_WINDOW (rt->dialog),
		&left, &top);
	GtkAllocation alloc = rt->textview->allocation;
	*x = left + alloc.x + alloc.width/2;
	*y = top + alloc.y + alloc.height/2;
	*push_in = TRUE;
}
/**
 * @brief construct and show output-pane context menu
 *
 * @param textview the textview widget where the click happened
 * @param event_button which mouse button was clicked (0 for a menu key)
 * @param event_time time that the event happened (0 for a menu key)
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_view_dialog_show_context_menu (GtkWidget *textview,
	guint event_button, gint event_time, E2_ViewDialogRuntime *rt)
{
	gchar *item_name;
	GtkWidget *menu = gtk_menu_new ();

//	GtkWidget *item = 
//	e2_menu_add (menu, _("_Find"), GTK_STOCK_FIND,
//		NULL,  _e2_view_dialog_menu_find_cb, rt);
//	gtk_tooltips_set_tip (app.tooltips, item,
//		_("Show the search options bar"), _("Find the first/next occurrence"));

//	GtkWidget *submenu = e2_menu_add_submenu (menu, _("_Settings"));
//	e2_option_create_menu (GTK_WIDGET (textview), submenu,
//		opt_view_wrap, NULL, NULL,
// etc
//		app.output.opt_show_on_new, NULL, NULL,
//		app.output.opt_show_on_focus_in, NULL, NULL,
//		NULL);
//	e2_menu_add_separator (submenu);
	item_name = g_strconcat (_A(2),".",_A(30),NULL);
	e2_menu_add_action (menu, _("_Settings"), GTK_STOCK_PREFERENCES,
		_("Open the configuration dialog at the view options page"),
		item_name, _C(40), //_("view")
		NULL);
	g_free(item_name);

	g_signal_connect (G_OBJECT (menu), "selection-done",
		G_CALLBACK (e2_menu_destroy_cb), NULL);

	if (event_button == 0)
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
			(GtkMenuPositionFunc) e2_view_dialog_set_menu_position,
			rt, event_button, event_time);
	else
		//this was a button-3 click
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
			NULL, NULL, event_button, event_time);
}
/**
 * @brief construct and show hbox with search-bar items
 *
 * @param rt runtime struct to work on
 *
 * @return hbox for the searchbar
 **/
GtkWidget *e2_view_dialog_create_searchbar (E2_ViewDialogRuntime *rt)
{
	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);

	//add search-string combo
	rt->combo = e2_combobox_add (hbox,
		FALSE, 0, NULL, rt,
		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
	gtk_widget_set_size_request (rt->combo, 230, -1);
	e2_combobox_append_history (rt->combo, find_history);
	e2_view_dialog_update_combo (rt->combo);

	GtkWidget *entry = GTK_BIN (rt->combo)->child;
	g_signal_connect_after (G_OBJECT (entry), "key_release_event",
		G_CALLBACK (e2_view_dialog_combokey_cb), rt);
	e2_widget_set_tooltip (NULL, entry, _("Finds"));
	//add search-option buttons
	GtkWidget *btn = e2_button_add_toggle (hbox, TRUE, case_sensitive,
		_("_match case"), _("If activated, text case does matter when searching"),
		case_sensitive, 2, e2_view_dialog_toggled, &case_sensitive);
	btn = e2_button_add_toggle (hbox, TRUE, search_backward,
		_("_backward"),_("If activated, searching proceeds toward document start"),
		search_backward, 2, e2_view_dialog_toggled, &search_backward);
	btn = e2_button_add_toggle (hbox, TRUE, search_wrap,
		_("_loop"), _("If activated, searching cycles from either end to the other"),
		search_wrap, 2, e2_view_dialog_toggled, &search_wrap);

	gtk_widget_show (hbox);
	return hbox;
}
/**
 * @brief setup initial view position, when re-viewing/editing a file
 *
 * @param filepath utf8 string with full path of item being re-opened
 * @param rt runtime struct to work on
 *
 * @return hbox for the searchbar
 **/
void e2_view_dialog_show_atlast (gchar *filepath, E2_ViewDialogRuntime *rt)
{
	GtkTextIter top;
	GList *iterator;
	E2_ViewHistory *viewed;
	for (iterator = view_history; iterator != NULL; iterator = iterator->next)
	{
		viewed = (E2_ViewHistory *) iterator->data;
		if (g_str_equal (viewed->filepath, filepath))
			break;
	}
	if (iterator != NULL)
	{
		//cannot use buffer coordinates here (bad for all but smallest files)
		gtk_text_buffer_get_iter_at_line (rt->textbuffer, &top, viewed->topline);
		//need to add a mark to get the scroll to work properly
		GtkTextMark *mark = gtk_text_buffer_create_mark
			(rt->textbuffer, NULL, &top, FALSE);
		gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (rt->textview), mark,
			0.0, TRUE, 0.0, 0.0);
	}
	else
		gtk_text_buffer_get_start_iter (rt->textbuffer, &top);

	gtk_text_buffer_place_cursor (rt->textbuffer, &top);

	e2_dialog_show (rt->dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL,
		NULL);
	gtk_window_present (GTK_WINDOW (rt->dialog));
}

void e2_view_dialog_destroy (E2_ViewDialogRuntime *rt)
{
	//in case we last performed an incremental search ...
	if (GTK_WIDGET_VISIBLE (rt->panel))
	{
		GtkWidget *entry = GTK_BIN (rt->combo)->child;
		const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
		if (*find != '\0')
		{
			e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
			e2_list_update_history (find, &find_history, NULL, 30, FALSE);
		}
	}
	//in case we want to re-view, remember where we are now
	//(need to use line no. as buffer coords are no use when reloading)
	GList *iterator;
	E2_ViewHistory *viewed;
	for (iterator = view_history; iterator != NULL; iterator = iterator->next)
	{
		viewed = (E2_ViewHistory *) iterator->data;
		if (g_str_equal (viewed->filepath, rt->filepath))
			break;
	}
	if (iterator == NULL)
	{
		viewed = (E2_ViewHistory *) g_malloc (sizeof (E2_ViewHistory));
		viewed->filepath = g_strdup (rt->filepath);
		view_history = g_list_append (view_history, viewed);
	}
	
	GdkRectangle visible_rect;
	GtkTextIter start;
	gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (rt->textview), &visible_rect);
	gtk_text_view_get_line_at_y (GTK_TEXT_VIEW (rt->textview), &start, 
		visible_rect.y, NULL);
//	if ()
//	{
		//when editing the output buffer, we don't want to change it
		GtkTextIter scan;
		guint i = 0;
		do
		{
			gtk_text_buffer_get_iter_at_line (rt->textbuffer, &scan, i++);
		} while (gtk_text_iter_compare (&scan, &start) < 0);
		viewed->topline = i-1;
/*	}
	else
	{
		//get gtk to count the preceding lines by deleting the rest of the conents
		GtkTextIter end;
		gtk_text_buffer_get_end_iter (rt->textbuffer, &end);
		gtk_text_buffer_delete (rt->textbuffer, &start, &end);
		viewed->topline = gtk_text_buffer_get_line_count (rt->textbuffer);
		if (viewed->topline > 0)
			viewed->topline--;
	} */

#ifdef E2_TRANSIENTKEYS
	e2_keybinding_unregister (rt->key_binding, GTK_WIDGET (rt->textview),
	g_free (rt->key_binding);
#endif
	gtk_widget_destroy (rt->dialog);
	g_free (rt->filepath);
	if (rt->idle_id != 0)
		g_source_remove (rt->idle_id);
	g_free (rt);
}

  /*********************/
 /***** callbacks *****/
/*********************/

/**
 * @brief menu-button press callback
 *
 * @param textview the textview widget where the press happened
 * @param rt dialog runtime data struct
 *
 * @return TRUE always
 **/
static gboolean _e2_view_dialog_popup_menu_cb (GtkWidget *textview,
	E2_ViewDialogRuntime *rt)
{
	gint event_time = gtk_get_current_event_time ();
	_e2_view_dialog_show_context_menu (textview, 0, event_time, rt);
	return TRUE;
}
/**
 * @brief mouse button press callback
 *
 * @param textview the widget where the button was pressed
 * @param event gdk event data
 * @param rt rt data for the dialog
 *
 * @return TRUE (stop other handlers) for btn 3 press, else FALSE (allow other handlers)
**/
static gboolean _e2_view_dialog_button_press_cb (GtkWidget *textview, 
	GdkEventButton *event, E2_ViewDialogRuntime *rt)
{
	if (event->button == 3)
	{
		_e2_view_dialog_show_context_menu (textview, event->button, event->time, rt);
		return TRUE;
	}
	return FALSE;
}
/**
 * @brief check-button state change callback
 *
 * @param button UNUSED the button widget that changed
 * @param store pointer to store for the new state of @a button
 * 
 * @return
**/
void e2_view_dialog_toggled (GtkWidget *button, gboolean *store)
{
	*store = GTK_TOGGLE_BUTTON (button)->active;
}
/**
 * @brief general dialog key press callback
 *
 * @param textview UNUSED the widget where the button was pressed
 * @param event gdk event data
 * @param rt rt data for the dialog
 *
 * @return TRUE (stop other handlers) for recognised keys
**/
gboolean _e2_view_dialog_key_press_cb (GtkWidget *textview, 
	GdkEventKey *event, E2_ViewDialogRuntime *rt)
{
	guint modifiers = gtk_accelerator_get_default_mod_mask ();
	modifiers = event->state & modifiers;
	if (//g_ascii_isalpha (event->keyval) //&& (event->keyval < 0xF000) //&&
		(modifiers == GDK_CONTROL_MASK || modifiers == GDK_MOD1_MASK))
	{	// the key is a letter and the modifier is Control or Alt
		if (event->keyval == find_keycode
			|| event->keyval == GDK_g)	//special case, not in button label
		{
			_e2_view_dialog_response_cb (rt->dialog, E2_RESPONSE_FIND, rt);
			return TRUE;
		}
		else if (event->keyval == hide_keycode)
		{
			_e2_view_dialog_response_cb (rt->dialog, E2_RESPONSE_USER3, rt);
			return TRUE;
		}
	}
	return FALSE;
}
/**
 * @brief search string change (key-press) callback
 *
 * This generally implements incremental searches,
 * except when the Enter key is pressed.
 *
 * @param entry UNUSED the entry widget where the key was pressed
 * @param event pointer to event data struct
 * @param view rt data for the search
 * 
 * @return FALSE always
**/
gboolean e2_view_dialog_combokey_cb (GtkWidget *entry,
	GdkEventKey *event, E2_ViewDialogRuntime *rt)
{
	guint modifiers = gtk_accelerator_get_default_mod_mask ();
	modifiers = event->state & modifiers;
	//except for Return key, incremental search is ineffective
	if (event->keyval == GDK_Return)
	{
		if (modifiers == GDK_SHIFT_MASK)
		{	//temporary direction change
			search_backward = !search_backward;
			e2_view_dialog_search (FALSE, FALSE, rt);
			search_backward = !search_backward;
		}
		else if (modifiers == GDK_CONTROL_MASK)
			e2_view_dialog_search (TRUE, FALSE, rt);
		else
			e2_view_dialog_search (FALSE, FALSE, rt);
	}
	else if (event->keyval < 0xF000)	//only interested in "text" keypresses
	{	//we recognise the equivalent of button presses here
		if (modifiers == GDK_CONTROL_MASK || modifiers == GDK_MOD1_MASK)
		{
			if (event->keyval == find_keycode
				|| event->keyval == GDK_g)	//special case, not in button label
			{
				_e2_view_dialog_response_cb (rt->dialog, E2_RESPONSE_FIND, rt);
				return TRUE;
			}
			else if (event->keyval == hide_keycode)
			{
				if (rt->replacebar != NULL)	//actually in edit mode
					return (e2_edit_dialog_key_press_cb (rt->textview, event, rt));
				else
				{
					_e2_view_dialog_response_cb (rt->dialog, E2_RESPONSE_USER3, rt);
					return TRUE;
				}
			}
			else if (event->keyval == replace_keycode)
			{	//this combo is used for the edit dialog too
				if (rt->replacebar != NULL)	//actually in edit mode
					return (e2_edit_dialog_key_press_cb (rt->textview, event, rt));
			}
		}
		// by default, do an incremental search
		e2_view_dialog_search (FALSE, TRUE, rt);
	}
	return FALSE;
}
/**
 * @brief view dialog response callback
 *
 * @param dialog the dialog widget where the response was triggered
 * @param response the number assigned the activated widget
 * @param view rt data for the dialog
 * 
 * @return
**/
static void _e2_view_dialog_response_cb
	(GtkWidget *dialog, gint response, E2_ViewDialogRuntime *rt)
{
	printd (DEBUG, "response_cb (dialog:_,response:%d,rt:_)", response);
	switch (response)
	{
		case E2_RESPONSE_USER1:  //text wrap
			rt->textwrap = !rt->textwrap;
			gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
				rt->textwrap ? GTK_WRAP_WORD: GTK_WRAP_NONE);
			break;
		case E2_RESPONSE_FIND:
			if (GTK_WIDGET_VISIBLE (rt->panel)) //search panel is visible
			{	//if a mod key is pressed, treat it specially CANT, no response
				//Ctrl = first, Shift = temp reverse
/*				GdkDisplay *display =
				gdk_display_manager_get_default_display
					(gdk_display_manager_get());
				GdkModifierType mask;
				gdk_display_get_pointer (display, NULL, NULL, NULL, &mask);
				if (mask & GDK_SHIFT_MASK)
				{	//temporary direction change
					search_backward = !search_backward;
					_e2_view_dialog_search (FALSE, FALSE, rt);
					search_backward = !search_backward;
				}
				else if (mask & GDK_CONTROL_MASK)
					_e2_view_dialog_search (TRUE, FALSE, rt);
				else */
					e2_view_dialog_search (FALSE, FALSE, rt);
			}
			else
			{	//show the search panel
				gtk_widget_show (rt->panel);
				//show extra buttons (if we're in a view dialog as opposed to edit)
//				gtk_widget_show (rt->nextbtn);
				if (rt->hidebtn != NULL)
					gtk_widget_show (rt->hidebtn);
//				gtk_widget_hide (rt->refreshbtn);
				//focus on the text entry
				gtk_widget_grab_focus (GTK_BIN (rt->combo)->child);
				//swap find btn tip
				if (rt->findbtn != NULL)
					e2_widget_swap_tooltip (rt->findbtn);
			}
			break;
/*		case E2_RESPONSE_USER2:	//find next
			_e2_view_dialog_search (FALSE, FALSE, rt);
			break; */
		case E2_RESPONSE_USER3:	//hide
		{
			//in case we last performed an incremental search ...
			GtkWidget *entry = GTK_BIN (rt->combo)->child;
			const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
			if (*find != '\0')
			{
				e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
				e2_list_update_history (find, &find_history, NULL, 30, FALSE);
			}
			gtk_widget_hide (rt->panel);
//			gtk_widget_hide (rt->nextbtn);
			if (rt->hidebtn != NULL)	//in view mode
				gtk_widget_hide (rt->hidebtn);
			gtk_widget_hide (rt->info_label);
//			gtk_widget_show (rt->refreshbtn);
			//revert find btn tip
			if (rt->findbtn != NULL)
				e2_widget_swap_tooltip (rt->findbtn);
			gtk_widget_grab_focus (rt->textview);
		}
			break;
		default:
			e2_view_dialog_destroy (rt);
			return;
			break;
	}
}

#ifdef E2_TRANSIENTKEYS
/**
 * @brief function to setup default key-bindings for view dialog
 *
 * @param set pointer to option data struct
 *
 * @return
**/
static void _e2_view_dialog_keybindings (E2_OptionSet *set)
{
	//the key name strings are parsed by gtk, and except for simple letters
	//(which might reasonably change for different languages) no translation
	//is possible
	e2_option_tree_setup_defaults (set,
	g_strdup("keybindings=<"),  //internal name
	//the category string(s) here need to match the binding name
//	g_strconcat(_C(16),"||||",NULL),  //_("general"
	g_strconcat(_C(9),"||||",NULL),  //_("dialogs"
	g_strconcat("\t",_A(42),"||||",NULL),  //_("edit"
	g_strconcat("\t\t|<Control>j","||",_A(1),".",_A(2),"|ARG",NULL),
	g_strconcat("\t\t|<Control>k","||command|ARG",NULL),
	g_strdup(">"),
	NULL);
}
#endif

  /***********************/
 /***** dialog setup *****/
/***********************/
/**
 * @brief create and show view dialog
 *
 * @param filename utf8 string which has path & name of item to be processed
 * @param srt pointer to store for returning dialog data, or NULL
 * 
 * @return the dialog widget, or NULL if there's a problem
**/
static GtkWidget *_e2_view_dialog_create (gchar *filename, 
	E2_ViewDialogRuntime **srt)
{
	//init view runtime object (0's to ensure edit-dialog things are NULL)
	E2_ViewDialogRuntime *rt = g_malloc0 (sizeof (E2_ViewDialogRuntime));
	rt->filepath = filename;	//interim store, for the file-read func
	rt->textview = gtk_text_view_new ();
	// add the item's content
	if (!e2_view_dialog_read_text (rt))
	{
		gtk_widget_destroy (rt->textview);
		g_free (rt);
		if (srt != NULL)
		*srt = NULL;
		return NULL;
	}
	rt->dialog = e2_dialog_create (NULL, rt->filepath, _("displaying file"),
		_e2_view_dialog_response_cb, rt);
	gtk_window_set_type_hint (GTK_WINDOW (rt->dialog),
			    GDK_WINDOW_TYPE_HINT_NORMAL);
	//create the view
	GtkWidget *sw = e2_widget_add_sw (GTK_DIALOG (rt->dialog)->vbox,
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, E2_PADDING);
	gtk_container_add (GTK_CONTAINER (sw), rt->textview);
	gtk_widget_show (rt->textview);
	// set view defaults
	gtk_text_view_set_editable (GTK_TEXT_VIEW (rt->textview), FALSE);
	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (rt->textview), FALSE);
	rt->textwrap = e2_option_bool_get ("dialog-view-wrap");
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
		rt->textwrap ? GTK_WRAP_WORD : GTK_WRAP_NONE);
	gint char_width, char_height;
	const gchar *fontstr = e2_utils_get_output_font ();
	e2_widget_set_font (rt->textview, fontstr, &char_width, &char_height);
	rt->window_width = e2_option_int_get ("dialog-view-width");
	rt->window_height = e2_option_int_get ("dialog-view-height");;
	rt->idle_id = 0;
	//init search runtime data
	rt->history = e2_list_copy_with_data (find_history);

	//action area is a GtkHButtonBox, things can't be stacked
	//vertically there
	//so we put the search panel at the bottom of the dialog's vbox
	rt->panel = e2_widget_add_box (GTK_DIALOG (rt->dialog)->vbox,
		FALSE, 0, TRUE, FALSE, 0);
	gtk_widget_hide (rt->panel);
	//add handlebox
	GtkWidget *hndlbox = gtk_handle_box_new ();
	gtk_handle_box_set_shadow_type (GTK_HANDLE_BOX (hndlbox),
		GTK_SHADOW_NONE);
	gtk_handle_box_set_handle_position (GTK_HANDLE_BOX (hndlbox),
		GTK_POS_LEFT);
	gtk_container_add (GTK_CONTAINER (rt->panel), hndlbox);
	gtk_widget_show (hndlbox);
	GtkWidget *hbox = e2_view_dialog_create_searchbar (rt);
	gtk_container_add (GTK_CONTAINER (hndlbox), hbox);
//now add things to the action-area
	//the "not found" warning is created but not displayed
	gchar *labeltext = g_strconcat ("<span weight=\"bold\" foreground=\"",
		e2_option_str_get ("color-negative"), "\">", _("not found"), "</span>", NULL);
	rt->info_label =
		e2_widget_add_label (GTK_DIALOG (rt->dialog)->action_area, labeltext, 0.0, TRUE, 0);
	//left-align the label
	gtk_button_box_set_child_secondary (
		GTK_BUTTON_BOX (GTK_DIALOG (rt->dialog)->action_area), rt->info_label, TRUE);
	gtk_widget_hide (rt->info_label);
	//search and/or view buttons
	labeltext = _("_Hide");
	rt->hidebtn = e2_dialog_add_undefined_button
		(rt->dialog, GTK_STOCK_ZOOM_FIT, labeltext, E2_RESPONSE_USER3);
	e2_widget_set_tooltip (NULL, rt->hidebtn, _("Hide the search options bar"));
	hide_keycode = e2_utils_get_mnemonic_keycode (labeltext);
	//this one not seen yet
	gtk_widget_hide (rt->hidebtn);

	e2_dialog_add_check_button (rt->dialog, rt->textwrap, _("_wrap"),
		_("If activated, text in the window will be word-wrapped"),
		E2_RESPONSE_USER1);

	labeltext = _("_Find");
	rt->findbtn = e2_dialog_add_undefined_button
		(rt->dialog, GTK_STOCK_FIND, labeltext, E2_RESPONSE_FIND);
	gtk_tooltips_set_tip  (app.tooltips, rt->findbtn,
		_("Show the search options bar"), _("Find the next match"));
	find_keycode = e2_utils_get_mnemonic_keycode (labeltext);

	//for a view dialog, we never want to match replace keypresses
	replace_keycode = 0;

	e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_CLOSE);
//	e2_dialog_set_responses (rt->dialog, E2_RESPONSE_FIND, GTK_RESPONSE_CLOSE);
	e2_dialog_set_negative_response (rt->dialog, GTK_RESPONSE_CLOSE);
	//this prevents a check button from being activated by keyboard
//	gtk_dialog_set_default_response (GTK_DIALOG (rt->dialog), E2_RESPONSE_FIND);

	gtk_window_resize (GTK_WINDOW (rt->dialog), char_width * rt->window_width,
				char_height * rt->window_height);

#ifdef E2_TRANSIENTKEYS
	//add dialog-specific key bindings, before the key-press callback
	//group name (must be freeable)
//	rt->key_binding = g_strconcat (_C(16), ".", _A(42), NULL);	//_(general.view
	rt->key_binding = g_strconcat (_C(9), ".", _A(42), NULL);	//_(dialogs.view
	e2_keybinding_register_transient (rt->key_binding, GTK_WIDGET (rt->textview),
		_e2_view_dialog_keybindings);
#endif

	g_signal_connect (G_OBJECT (rt->textview), "popup_menu",
		G_CALLBACK (_e2_view_dialog_popup_menu_cb), rt);
	g_signal_connect (G_OBJECT (rt->textview), "button_press_event",
		G_CALLBACK (_e2_view_dialog_button_press_cb), rt);
	g_signal_connect (G_OBJECT (rt->textview), "key_press_event",
		G_CALLBACK (_e2_view_dialog_key_press_cb), rt);

	if (srt != NULL)
		*srt = rt;	//make rt available to openat func

	return rt->dialog;
}
  /*******************/
 /***** actions *****/
/******************/
/**
 * @brief open file for viewing, at a location where there is a specified string
 * @a rt_data is a string with filepath and an argument that is a text string to find in the file
 * @param action_data NULL always (ptr to data assigned when action struct created at session start)
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @return
 **/
void e2_view_dialog_openat (gpointer action_data, gpointer rt_data)
{
	gchar *arg = (gchar *) rt_data;
	gchar *gap = e2_utils_find_whitepace (arg);	//always ascii ' ', don't need g_utf8_strchr()
	gchar c; //ignore compiler warning about unitialized usage
	if (gap != NULL)
	{
		c = *gap;
		*gap = '\0';
	}

	E2_ViewDialogRuntime *srt;
	GtkWidget *dialog = _e2_view_dialog_create (arg, &srt);
	if (dialog != NULL)
	{
		gchar *locator;
		if (gap == NULL)
			locator = NULL;
		else
			locator = e2_utils_pass_whitepace (gap+1);
		if (locator != NULL && *(locator+1) != '\0')
		{	//find the locator and open there
			GtkTextIter iter, start, end;
			gtk_text_buffer_get_start_iter (srt->textbuffer, &iter);
			if (gtk_text_iter_forward_search (&iter, locator, 0, &start, &end, NULL))
			{	//need to add a mark to get the scroll to work properly
				GtkTextMark *mark = gtk_text_buffer_create_mark
					(srt->textbuffer, NULL, &start, FALSE);
				gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (srt->textview), mark,
					0.0, TRUE, 0.0, 0.1);
				//to support incremental searching, move the cursor
				//to the start of the located string
				gtk_text_buffer_place_cursor (srt->textbuffer, &start);
			}
		}
		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL,
			NULL);
		gtk_window_present (GTK_WINDOW (dialog));
	}
	if (gap != NULL)
		*gap = c;	//revert the original string, ready for next time
}
/**
 * @brief re-open file for viewing, at the last-used location in the file
 * @param action_data NULL always (ptr to data assigned when action struct created at session start)
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @return
 **/
gboolean e2_view_dialog_reopen (gpointer action_data, gpointer rt_data)
{
	//process the 1st selected item in active pane
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	gchar *filename = (*(E2_SelectedItemInfo **)names->pdata)->filename;
	E2_ViewDialogRuntime *srt;
	GtkWidget *dialog = _e2_view_dialog_create (filename, &srt);
	if (dialog != NULL)
	{
		//for name comparisons, we need the full path
		filename = g_strconcat (curr_view->dir, filename, NULL);
		e2_view_dialog_show_atlast (filename, srt);
		g_free (filename);
	}
	e2_fileview_clean_selected (names);
	return TRUE;
}

  /******************/
 /***** public *****/
/******************/

//filename is utf8
void e2_view_dialog_show (gchar *filename)
{
	GtkWidget *dialog = _e2_view_dialog_create (filename, NULL);
	if (dialog != NULL)
	{
		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
		gtk_window_present (GTK_WINDOW (dialog));
	}
}

void e2_view_dialog_actions_register (void)
{
	//HACK while we're at it, init some session parameters
	//these options are static for the session
	case_sensitive = e2_option_bool_get ("dialog-search-case-sensitive");
	show_last = e2_option_bool_get ("dialog-search-show-last");

	if (e2_option_bool_get ("dialog-search-history"))
		e2_cache_list_register ("search-history", &find_history);

	gchar *action_name = g_strconcat(_A(5),".",_A(87),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		e2_view_dialog_reopen, NULL, FALSE);
	action_name = g_strconcat(_A(5),".",_A(88),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		e2_view_dialog_openat, NULL, TRUE);

	//FIXME put this somewhere more sensible
	action_name = g_strconcat(_A(5),".",_A(37),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		e2_edit_dialog_reopen, NULL, FALSE);
}

void e2_view_dialog_options_register (void)
{
	gchar *group_name = g_strconcat(_C(9),":",_C(40),NULL); //_("dialogs:view"
	//first some options that may, but probably won't, change during the session
	e2_option_bool_register ("dialog-view-wrap",
		group_name, _("wrap text"),
		_("This causes the view window to open with text-wrapping enabled"),
		NULL, TRUE,
		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREEGROUP);
	e2_option_int_register ("dialog-view-width",
		group_name, _("window width"),
		_("The view window will default to showing this many characters per line (but the the displayed buttons may make it wider than this)")
		, NULL, 84, 20, 1000,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_int_register ("dialog-view-height",
		group_name, _("window height"),
		_("The view window will default to showing this many lines of text"), NULL, 30, 10, 1000,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register
		("dialog-search-case-sensitive", group_name, _("case sensitive searches"),
		_("This causes the view window search-bar to first open with case-sensitive searching enabled"),
		NULL, FALSE,
		E2_OPTION_FLAG_BASIC);
	e2_option_bool_register ("dialog-search-show-last",
		group_name, _("show last search string"),
		_("This shows the last search-string in the entry field, when the view window search-bar is displayed"),
		NULL, TRUE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register
		("dialog-search-history", group_name, _("keep search history"),
		_("This causes search strings to be remembered between sessions"),
		NULL, FALSE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register ("use-external-encoder", group_name, _("use external encoding converter"),
		_("If activated, the command entered below will be run, instead of using the internal conversion functions, to convert file character encoding when needed"),
		NULL, FALSE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_str_register ("external-encoder", group_name, _("converter command:"),
		_("A command which runs an external application to convert text encoding to UTF-8"),
		"use-external-encoder", "enca -x UTF-8",
		E2_OPTION_FLAG_ADVANCED);

	//FIXME put this somewhere more sensible
	group_name = g_strconcat(_C(9),":",_A(36),NULL); //_("dialogs:edit"
	//first some options that may, but probably won't, change during the session
	e2_option_bool_register ("edit-save-backup", group_name, _("backup when saving"),
		_("When saving an edited file, an existing file with the same name will be renamed"),
		NULL, FALSE,
		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREEGROUP);
}
