/* $Id: e2_main.c 1872 2006-04-27 09:41:59Z tpgww $
 *
 * Copyright (C) 2003-2006 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file src/e2_main.c 
 * @brief main function
 *
 * This file contains the main function and some local startup and shutdown helpers.
 **/

/*! \mainpage emelFM2 API
  *
	* \attention This API-documentation is under construction!\n Plans are to 
	* provide you with additional information to make it easier to read the 
	* code and hopefully inspire you to enhance it. For now you will not find 
	* much and maybe outdated information as emelFM2 changes quickly. 
	* This should change with version 0.1.0.
  *
  * \section intro introduction
  *
  * Welcome to emelFM2-doxygen. On this page you will find all the important 
	* aspects of coding in emelFM2. It should provide you with better 
	* understanding of the code. There are three parts:\n
	* \arg \ref conventions
	* \arg \ref internals
	* \arg \ref visible\n
	*
	*
	* \section conventions conventions
	*
	* emelFM2-code should follow some layout and naming conventions:
	* \arg Use tabs, not spaces, to indent code. (Recommended is a tab size of 4.)
	* \arg Stick to the same consistant coding style (vertically aligned braces; 
	* a space after if, while, functions, etc.; "//" C++ style comments):
	* \code
	* void routine (void)
	*	{
	*		if (function (a, b, c))
	*		{
	*			//do something
	*			x = a + 1;
	*			printf (_("Break lines after 80 characters, in general, like the following.\n))
	*			rt->set = e2_option_tree_register ("option_name", strdup("group_name"), _("option_label"),
	*			NULL, NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL);
	*			printf (_("or _perhaps_ NOT after 80, if the line is just a string that needs to be internationalised like this.\n"));
	*		}
	*	}
	* \endcode
	* \arg emelFM2's own functions start with e2_ 
	* \arg 'private' functions begin with _e2 - those should not be used 
	* from other parts of the programm
	* \arg emelFM2's own constants begin with E2_
	* \arg files coded for emelFM2 begin with e2_
	* \arg all functions not beginning with e2_ are either gtk_, g_ 
	* or old one's from emelFM. The latter ones will be replaced in the future.
	* \arg .h files can included with just \"\" even if they are not in the 
	* same directory
	* 
	* If you don't know doxygen well: try to click on everything you are 
	* interested. Even within the source documentation you will find lots 
	* of links. It's quite fun.
	* 
	* \section internals emelFM2 internals
	*
	* This section explains how emelFM2 works inside.
	* \arg \ref option_sys
	* \arg \ref cache
	* \arg \ref action_sys
	* \arg \ref virtual_fs
	* \arg \ref treeview
	* \arg \ref plugins
	*
	* \subsection option_sys the option system
	*
	* The option system is initialized in \ref e2_main.c. It is first 
	* loaded with some dummy values. After all other initializations are 
	* finished those dummy-data will be replaced with input from the 
	* 'config' file.  FIX THIS
	*
	* \subsection cache the cache
	*
	* ToDo
	*
	* \subsection action_sys the action system
	*
	* ToDo
	*
	* \subsection virtual_fs the virtual filesystem
	*
	* Not implemented yet.
	*
	* \subsection treeview treeview
	*
	* ToDo
	*
	* \subsubsection cell_rend cell-renderers
	*
	* ToDo
	*
	* \subsection plugins plugins
	*
	* \attention Due to the early stage of development of emelFM2, 
	* the plugin api will probably change. So the following information
	* is subject to change.
	*
	* Here is an outline of the Plugin Writing process.\n\n
	* <em>Step 1)</em> \code#include "emelfm2.h"\endcode You need this to 
	* get access to all the functions and data structures of emelFM2\n\n
	* <em>Step 2)</em> Write the function that does what you want your plugin to 
	* do when activated. For example, if you want a plugin that prints 
	* "Hello World", then you would write this function.
	* \code void hello_world(void)
	* {  
	* 	status_message(_("Hello World\n)");
	* }
	* \endcode The restrictions on this function are that it must return 
	* void and can take no parameters.\n\n
	* <em> Step 3)</em> Write an initialisation function something like this:
	* \code
	* //aname must be confined to this module
    * static gchar *aname; //a translated component of the action-name for the plugin
	* gboolean init_plugin (Plugin *p)
	* {
    *  aname = _("demonstration");
	* #define ANAME ""HelloWorld"  //part of the registered action name, no spaces, not translated
    *  p->signature = ANAME VERSION;  //for detecting whether the plugin is loaded
    * 	p->menu_name = _("_Hello World");  //the name for the plugins menu item, capitalize according to HIG is best
	* 	p->description = _("prints \"Hello World\" on the output window");  //the tooltip for the plugins menu item
	* 	p->icon = "plugin_"ANAME"_48.png";  //a non-standard path may be prepended. Just put "" for no icon
	*
	*  //sometimes we load the plugin to get the data above, but don't want to use it ...
	* if (p->plugin_cb == NULL)
	* {  //but mostly we want to set it up for usage ...
	*	  p->plugin_cb = hello_world;  //register the function that the plugins menu item will call 
	* 					// (the name is whatever you named the function in Step 2)
	*	gchar *action_name = g_strconcat (PLUGIN,"."aname,NULL); //no need to free this
	*	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, hello_world, NULL, FALSE);
	*
	*	//other initialization stuff, as appropriate ...
	*
	*	return TRUE; //or FALSE, if the initialisation failed
	* }
	*  return FALSE;
	* }\endcode
	*	
	* This function \b must be called \c init_plugin, take a <tt>Plugin * </tt>argument, and 
	* return \c gboolean which indicates whether the initialisation succeeded.\n\n
	* <em> Step 4) </em>Write an unload_plugin function something like this:
	* \code void unload_plugin (Plugin *p)
	* {
	*	 gchar *action_name = g_strconcat (PLUGIN,"."aname,NULL);
	*   e2_action_unregister (action_name);
	*	g_free (action_name);
	* }
	*  //other stuff, as appropriate ...
	*}\endcode
	* This function \b must be called \c unload_plugin, take a <tt>Plugin *</tt> argument, 
	* and return \c void.
	* \n\n
	* \section visible emelFM2 visible
	*
	* This section is about parts of emelFM2 you can see and which interact
	* with the user. The following items are covered:
	* \arg \ref panes
	* \arg \ref toolbar
	* \arg \ref commline
	* \arg \ref menu
	* 
	* \subsection panes the panes
	*
	* ToDo
	*
	* \subsection toolbar the toolbar
	*
	* ToDo
	*
	* \subsection commline the commandline
	*
	* ToDo
	*
	* \subsubsection output_panel the output pane
	*
	* ToDo
	*
	* \subsection menu the menu
	*
	* ToDo
	*
  */

#include "emelfm2.h"
#include <string.h>
#include <unistd.h>
#include "e2_dialog.h"
#include "e2_filelist.h"
#include "e2_filetype.h"
#include "e2_plugins.h"
#include "e2_task.h"

extern gint refresh_ref;
#if defined (E2_CURRENTGTK_VERSION) && GLIB_CHECK_VERSION (2, 10, 0)
extern GHashTable *actions_hash;
#endif

static void _e2_main_check_locale_broken (gchar *locale)
 {
	gboolean broken = FALSE;
	if (g_str_equal (locale, "C"))
		broken = TRUE;
	else
	{
		gchar **split = g_strsplit (locale, ";", -1);
		gint i = 0;
		while (split[i] != NULL)
		{
			if (g_str_has_prefix (split[0], "LC_CTYPE") &&
				g_str_has_suffix (split[0], "=C"))
			{
				broken = TRUE;
				break;
			}
			i++;
		}
	}
	if (broken)
 	{
 		printf (_("Your current locale is '%s'.\n"), locale);
		printf (
		  _("You have set the environment variable G_BROKEN_FILENAMES, which\n"
 			"causes GTK+ to convert filename encoding, from the one specified\n"
			"by the system locale, to UTF-8.\n"
			"However, you have not set a system locale. Please do so, by setting\n"
			"the environment variable LANG or LC_CTYPE!\n"));
 		if (!e2_cl_options.ignore_problems)
 		{
			printf (
			  _("(Note: There is a command line option -i/--ignore-problems, but use it\n"
				"at your own risk!)\n"));
 			exit (1);
 		} else
			printf (
			  _("%s will ignore locale problems when reading filenames because\n"
 				"--ignore-problems/-i has been set. This might result in segfaults and all\n"
 				"kind of problems. You really should set a system locale with the\n"
				"LANG or LC_CTYPE environment variable.\n"), PROGNAME);
 	}
}

static void _e2_main_check_locale (void)
{
	printd (DEBUG, "check_locale ()");
	gchar *locale = gtk_set_locale ();
	printd (NOTICE, "current locale is '%s'", locale);
	const gchar *broken =  g_getenv ("G_BROKEN_FILENAMES");
	if (broken != NULL)
		_e2_main_check_locale_broken (locale);

#ifdef ENABLE_NLS
	printd (DEBUG, "setting locale dir to '%s'", LOCALE_DIR);
	bindtextdomain (BINNAME, LOCALE_DIR);
	textdomain (BINNAME);
	bind_textdomain_codeset (BINNAME, "UTF-8");
#endif
}

//blind log handler to stop gtk warnings
static void blind_log_handler () { }

  /****************/
 /***** main *****/
/****************/

gint main (gint argc, gchar *argv[])
{
	//threads needed
	g_thread_init (NULL);
	gdk_threads_init ();	//setup gdk mutex
	//gtk initialization
	printd (DEBUG, "gtk init");
	gtk_init (&argc, &argv);

	e2_cl_option_process (argc, argv);
	_e2_main_check_locale ();
#ifndef FILES_UTF8ONLY
	e2_fs_check_coding ();	//setup for path/filename conversion if needed
#endif
	if (e2_cl_options.suppress_gtk_output)
		g_log_set_handler ("Gtk", G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL
			| G_LOG_FLAG_RECURSION, blind_log_handler, NULL);

	//some local init stuff
	gboolean confile_ok;
	gboolean confdir_ok = e2_option_set_config_dir ();  //set name for config dir, make it if not there already
	if (confdir_ok)
		e2_cache_init ();  //read and process cache file (if any) (before options init)
	e2_action_setup_labels ();  //need this before option_init, to avoid crashes when there is no config file
	e2_option_setup_labels ();
	//after labels & before options setup, setup data for referencing toolbars
	e2_toolbar_data_create ();

	e2_option_init ();  //always setup default options (before config read)

	if (confdir_ok) //read & process saved options (if any)
		confile_ok = e2_option_file_read ();
	else
		confile_ok = FALSE;

	if (confile_ok)
	{
		//ASAP after config settled, CHECKME custom dialog ok ?
#define UPGRADE_PNAME "e2p_upgrade.so"
		if (strcmp (app.cfgfile_version, VERSION RELEASE) < 0)
		{
			Plugin *p = e2_plugins_load1 (PLUGINS_DIR UPGRADE_PNAME);
			if (p != NULL)
			{
				if (!p->plugin_init (p))	//do any upgrades
					printd (ERROR, "Can't initialize upgrade plugin: "UPGRADE_PNAME);
				e2_plugins_unload1 (p, FALSE);
			}
			else
				printd (ERROR, "Can't find upgrade plugin "UPGRADE_PNAME", so can't upgrade the config file");
		}
		gchar **values = e2_list_to_strv (e2_cl_options.option_overrides);
		e2_option_read_array (values);
		g_strfreev (values);
	}

	//after config & updates, install default tree options where needed
	e2_option_tree_install_defaults ();

	//before plugins loaded, before bars/menus created
	e2_actions_init (); //setup action data stores & register most actions

	//check if all plugin data is in config already
//	E2_OptionSet *set = e2_option_get ("plugins");  //internal name
//	gboolean defer_plugs = set->ex.tree.found_in_config_file;
//	if (!defer_plugs)
	//if this is a fresh start,
	//do this before bars/menus created
	e2_plugins_add_all (); // load plugins specified in the config data

	//window creation includes dis/enable_refresh, so the refresh count
	//must start > 1, to prevent erroneous timer behaviour during
	//window creation. Set to real initial value (0) later in the startup
	refresh_ref = 10;

	app.tooltips = gtk_tooltips_new ();
	g_hook_list_init (&app.hook_pane_focus_changed, sizeof (GHook));
	e2_utils_update_gtk_settings ();

	//build GUI
	printd (DEBUG, "create main window");
	e2_window_create (&app.window);
	//show the window before the filelist creation (in case there's a big delay)
	gtk_widget_show (app.main_window);

#ifdef E2_FAM
	//before change dir, signal unknown state
	app.pane1.FAMreq = -1;
	app.pane2.FAMreq = -1;
	//hookup to a file monitor if possible
	e2_fs_FAM_connect ();
#endif
	//config checks may be disabled in threaded list creation
	//so the mechanism needs to be initialised beforehand
	e2_option_enable_config_checks ();
	//	printd (DEBUG, "init filters"); now cached
//	e2_filelist_initialize_filters (app.pane1.view);
//	e2_filelist_initialize_filters (app.pane2.view);

	e2_utils_goto_accessible_path (&app.pane2);
	//show pane 1 last, making it the default for session start
	e2_utils_goto_accessible_path (&app.pane1);

	//set refresh counter to its real initial value
	refresh_ref = 0;
	if (e2_option_bool_get ("auto-refresh"))
	{
		printd (DEBUG, "init auto-refresh");
		//normal (1sec) delay set during window build
		app.filelist_timer_id = g_timeout_add 
			(E2_FILESCHECK_INTERVAL, (GSourceFunc) e2_filelist_check_dirty, NULL);
	}
	//setup change-monitoring for config file, if appropriate
//	e2_option_enable_config_checks ();
	e2_window_enable_status_update (-1);

	gint pos = gtk_paned_get_position (GTK_PANED (app.window.output_paned));
	if (pos == 0)
	{
		gchar *command = g_strconcat(_A(1),".",_A(39),NULL);  //_("command.focus")
		e2_action_run_simple (command, NULL);
		g_free (command);
	}

	e2_filetype_add_all ();	//process filetypes into data structure
	e2_button_setup_labels ();

	e2_cache_list_register ("open-history", &open_history); //history for 'open with' command
	e2_cache_int_register ("config-width", &app.cfgdlg_width, -1);	//default to 'normal' window size
	e2_cache_int_register ("config-height", &app.cfgdlg_height, -1);
	e2_option_set_trash_dir ();

	GList *tmp;
	for (tmp = e2_cl_options.startup_commands; tmp != NULL; tmp = g_list_next (tmp))
		e2_command_run (tmp->data, E2_COMMAND_RANGE_DEFAULT);

	if (!e2_cl_options.no_detach
#ifdef DEBUG_MESSAGES
		&& (e2_cl_options.debug_level < 5)
#endif
	)
		daemon (1, 1);

	//get all actions, for help document
//	e2_action_list_all ();

	printd (DEBUG, "enter main loop");
	gtk_main ();
	exit (0);
}

  /******************************/
 /***** departure lounge *****/
/******************************/

/**
 * @brief session-end cleanups
 * @param saveconfig TRUE to save config data FIXME always do that
 * @param announce TRUE to print debug message FIXME
 * @return
 **/
static void _e2_main_closedown (gboolean saveconfig, gboolean announce)
{
#ifdef E2_FAM
	e2_fs_FAM_disconnect ();
#endif
#ifdef E2_IMAGECACHE
	e2_cache_image_clearall ();
#endif
	if (saveconfig)  //this is interim, for debugging cleanup
	{
		e2_option_file_write (NULL);
		e2_cache_file_write ();
		printd (DEBUG, "touch config dir");
		e2_fs_touch_config_dir (); //signal to the world that changes saved
	}
#if defined (E2_CURRENTGTK_VERSION) && GLIB_CHECK_VERSION (2, 10, 0)
	g_hash_table_destroy (actions_hash);	//need to deallocate the glib slices
	g_hash_table_destroy (options_hash);
#endif
/*	printd (DEBUG, "clean options");
	destroy_config ();  //why bother, if the kernel does it all anyway?
								//cache(s) @ cache or named (lists ??)
								//FIXME?? = all those non-constant config strings
	printd (DEBUG, "clean bookmarks");
	e2_bookmark_clean ();
	printd (DEBUG, "clean actions");
	e2_actions_clean ();
*/
	if (announce)
	{
		printd (DEBUG, "system-initiated shutdown");
		//etc
	}
//	printd (DEBUG, "gtk_main_quit ()");
	gtk_main_quit ();
//	printd (DEBUG, "exit (0)");
	exit (0);
}
/**
 * @brief user-initiated shutdown
 * @return
 **/
void e2_main_user_shutdown (void)
{
	gint running = e2_command_count_children ();
	if (running > 0)
	{
		gchar *prompt = g_strdup_printf (_("%d child process(es) are running"), running);
		DialogButtons retval = e2_dialog_warning (prompt);
		g_free (prompt);
		if (retval != OK)
			gtk_main ();//CHECKME ok to just create another level ?
	}
	_e2_main_closedown (TRUE, FALSE);
}
/**
 * @brief system-initiated shutdown
 * @return
 **/
void e2_main_winclose_shutdown (GtkWidget *widget,
	GdkEvent  *event, gpointer data)
{
	_e2_main_closedown (TRUE, TRUE);
}
