/* $Id: e2_pane.c 1861 2006-04-25 08:27:44Z tpgww $
 *
 * Copyright (C) 2004-2006 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 /**
 * @file src/e2_pane.c 
 * @brief pane creation and action functions
 *
 * includes actions on pane contents, but not on panes per se
 *
 **/

#include "emelfm2.h"
//#include <unistd.h>
#include "e2_pane.h"
#include "e2_action.h"
#include "e2_dialog.h"
#include "e2_context_menu.h"
#include "e2_filelist.h"

static void _e2_pane_change_dir (E2_PaneRuntime *rt,
	gchar *path, gboolean history, gboolean hook);
extern guint last_selected_rows;

  /*****************/
 /***** utils *****/
/*****************/
/**
 * @brief set filepanes' properties to indicate which one is active
 *
 * The "active-pane-signal" config option is checked, and: the column
 * headers will be re-coloured, or the filename column title will be
 * [un]bolded, or the treeviews' sensitivity will be toggled
 *
 * @return
 **/
void e2_pane_flag_active (void)
{
	E2_OptionSet *set = e2_option_get ("active-pane-signal");
	gint htype = e2_option_int_get_direct (set);
	switch (htype)
	{
		case 1:
		{  //set bold Filename-column header title
			gchar *new_title = gettext (e2_all_columns[FILENAME].title);
			gchar *new_title2 = g_strconcat ("<b>", new_title,"</b>", NULL);
			gtk_label_set_markup (curr_view->name_label, new_title2);
			g_free (new_title2);
			//set normal Filename-column header title
			gtk_label_set_text (other_view->name_label, new_title);
			break;
		}
		case 2:
		{  //make active pane sensitive
			gtk_widget_set_sensitive (curr_view->treeview, TRUE);
			//make inactive pane insensitive
			gtk_widget_set_sensitive (other_view->treeview, FALSE);
			break;
		}
		default:
		{
			//set all active columns' header color
			GdkColor *active_btncolor = e2_option_color_get ("color-active-pane");
			GList *base = gtk_tree_view_get_columns (GTK_TREE_VIEW (curr_view->treeview));
			GList *columns;
			for (columns = base; columns !=NULL; columns = columns->next)
				gtk_widget_modify_bg (((GtkTreeViewColumn *)columns->data)->button,
					GTK_STATE_NORMAL, active_btncolor);
			g_list_free (base);
			//revert all inactive columns' header color to default
			base = gtk_tree_view_get_columns (GTK_TREE_VIEW (other_view->treeview));
			for (columns = base; columns != NULL; columns = columns->next)
				gtk_widget_modify_bg (((GtkTreeViewColumn *)columns->data)->button,
					GTK_STATE_NORMAL, NULL);
			g_list_free (base);
			break;
		}
	}
}
/**
 * @brief timeout function for checking completion of fileview creation
 *
 * @param userdata data specified when the timer was initialised, here, always NULL, unused
 *
 * @return TRUE until update is completed, then FALSE
 */
/*gboolean _e2_pane_views_complete (gpointer userdata)
{
	if (curr_view->completed && other_view->completed)
	{
		e2_fileview_switch_views ();
//		gtk_widget_grab_focus (curr_pane->focus_widget);
		gtk_widget_grab_focus (curr_view->treeview);
		e2_hook_list_run (&app.hook_pane_focus_changed, curr_pane);
		return FALSE;
	}
	printd (DEBUG, "finished waiting for fileview completion");
	return TRUE;  //continue waiting
}
*/
/**
 * @brief create filters menu for showing in pane to which @a view belongs
 *
 * @param view the view data struct
 *
 * @return the menu widget
 **/
static GtkWidget *_e2_pane_create_filter_menu (FileView *view)
{
	GtkWidget *menu = gtk_menu_new ();
	view->check_name = e2_menu_add_check (menu, _("_Name filter"),
		view->name_filter.active, e2_name_filter_dialog_create_cb, view);
	view->check_size = e2_menu_add_check (menu, _("_Size filter"),
		view->size_filter.active, e2_size_filter_dialog_create_cb, view);
	view->check_date = e2_menu_add_check (menu, _("_Date filter"),
		view->date_filter.active, e2_date_filter_dialog_create_cb, view);
	e2_menu_add_separator (menu);
	view->check_dirs = e2_menu_add_check (menu, _("_Directories too"),
		view->filter_directories, e2_fileview_filter_dirs_cb, view);
	if (view->name_filter.active 
		|| view->size_filter.active
		|| view->date_filter.active)
	{
		e2_menu_add_separator (menu);
		e2_menu_add (menu, _("_Remove all filters"), NULL, NULL, 
			e2_fileview_remove_filters_cb, view);
	}

	g_signal_connect (G_OBJECT (menu), "selection-done",
		G_CALLBACK (e2_menu_destroy_cb), NULL);

	return menu;
}

  /*******************/
 /***** actions *****/
/*******************/

/**
 * @brief create and show filters menu in active pane
 * @return TRUE always
 **/
static gboolean _e2_pane_filters_show (void)
{
	FileView *view = curr_view;
	GtkWidget *menu = _e2_pane_create_filter_menu (view);
	guint32 time = gtk_get_current_event_time ();
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
		(GtkMenuPositionFunc) e2_fileview_set_menu_position,
			view->treeview, 0, time);
	return TRUE;
}
/**
 * @brief change active pane from the current one to the other one
 * This action also toggles the visible filepane, if only 1 is shown
 * @return
**/
void e2_pane_activate_other_action (void)
{
	//if showing only 1 filepane, show the other one now
	E2_WindowRuntime *wrt = &app.window;
	if (wrt->panes_paned_ratio < 0.001)
		e2_window_adjust_pane_ratio_action (wrt, "1");
	else if (wrt->panes_paned_ratio > 0.999)
		e2_window_adjust_pane_ratio_action (wrt, "0");
	e2_pane_activate_other ();
}

/**
 * @brief change active pane from the current one to the other one
 *
 * @return
**/
void e2_pane_activate_other (void)
{
//	printd (DEBUG, "e2_pane_activate_other ()");
	E2_PaneRuntime *tmp = other_pane;
	other_pane = curr_pane;
	curr_pane = tmp;
	//force a status-line update
	last_selected_rows = -1;
	//DEBUG try this
//	if (curr_view->completed && other_view->completed)
//	{
//		printd (DEBUG, "NOT waiting for fileview completion");
		e2_fileview_switch_views ();
//		gtk_widget_grab_focus (tmp->focus_widget);
		if (GTK_WIDGET_HAS_FOCUS (other_view->treeview))
			gtk_widget_grab_focus (curr_view->treeview);
		e2_hook_list_run (&app.hook_pane_focus_changed, tmp);
/*	}
	else
	{  //wait until all is done
		printd (DEBUG, "start waiting for fileview completion");
		g_timeout_add (200, _e2_pane_views_complete, NULL);
	} */
}
/**
 * @brief change the directory for a specified pane
 *
 * This is a wrapper for _e2_pane_change_dir (), with updqte of
 * pane history list and hooklist
 * It may be called in or out of gtk's thread-lock
 *
 * @param rt pointer to pane data struct, or NULL for current pane
 * @param path utf string containing path to directory to be opened
 *
 * @return
**/
void e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path)
{
	_e2_pane_change_dir (rt, path, TRUE, TRUE);
}
/**
 * @brief change directory displayed in a specified pane
 *
 * ~ at start of path is interpreted to $HOME
 * redundant separators are removed, and one is added to the end,
 * if not already there
 * Does nothing if the path string is empty, shows error msg if
 * path is unreachable
 * Session-start check for valid path done in main(), not here
 * The supplied path string is unchanged.
 * New path is stored in heap @ rt->path
 * New path is also stored in array @ rt->view->dir
 *
 * @param rt pointer to pane data struct, or NULL for current pane
 * @param path utf-8 string with the path to open
 * @param history TRUE to update pane history list
 * @param hook TRUE to run change-dir hook functions (if any)
 *
 * @return
 */
static void _e2_pane_change_dir (E2_PaneRuntime *rt, 
	gchar *path, gboolean history, gboolean hook)
{
	printd (DEBUG, "_e2_pane_change_dir (rt:,path:%s,update:%d,hook:%d)",
		path, history, hook);

	gchar *freeme;

	if (path == NULL)
		return;
	//work on currently active file pane if argument is NULL
	if (rt == NULL)
		rt = curr_pane;

	path = g_strdup (path);

	g_strstrip (path);

	if (*path == '\0')
	{
		//quick exit
		g_free (path);
		return;
	}
	else if (g_str_equal (path, "..")
		&& g_str_equal (rt->path, G_DIR_SEPARATOR_S))
		//ignore impossible change
		return;
	else if (path[0] == '~') //home dir check
	{
		freeme = path;
		if (path[1] == '\0')
		{
			path = g_strdup (g_get_home_dir ());
			g_free (freeme);
		}
		else
			if (path[1] == G_DIR_SEPARATOR)
		{
			path = g_strconcat (g_get_home_dir (), path + 1, NULL);
			g_free (freeme);
		}
	}

#ifndef FILES_UTF8ONLY
	if (!app.utf8_filenames)
	{
		//this test not needed with only-utf8 file coding
		if (! g_utf8_validate (path, -1, NULL))
		{
			freeme = path;
//#ifdef E2_CURRENTGTK_VERSION && GTK_CHECK_VERSION = (2, 6, 0)
			path = g_filename_display_name (path);
/*#else
			path = e2_utf8_filename_from_locale (path);
#endif */
			g_free (freeme);
		}
	}
#endif

	//these fns both ensure a trailing /, which gets into rt->path
	//(see below) and onto dir lines
	if (e2_option_bool_get_direct (rt->opt_transparent))
	{	//relative path segment(s) conversion
		//clean and convert the path string, if needed
		freeme = path;
		path = e2_utils_translate_relative_path (rt->path, path);
		g_free (freeme);
	}
	// otherwise, just make sure the user didn't enter a 'dirty' path string
	else
		path = e2_utils_path_clean (path);

	if (! e2_fs_cd_isok (path) //can't go there
		|| ! e2_fs_chdir (path)) //actual go-there failed
	{
		g_free (path);
		return;
	}

	//create the new filelist (possibly threaded)
	e2_fileview_change_dir (rt->view, path);	//the new path

#ifdef E2_FAM
	gchar *oldpath = rt->path;
	rt->path = path;
	//startup alteration-monitoring for the new dir
	//(or if need be, for its target if it's a link)
	e2_fs_FAM_change (oldpath, rt);
	g_free (oldpath);
#else
	//update the path store
	g_free (rt->path);
	rt->path = path;
#endif	//def E2_FAM

	if (history)
	{	//update the pane forward-back list
		if (rt->history_cur < g_list_length (rt->history))
		{
			GList *free = g_list_nth (rt->history, rt->history_cur);
			GList *last = g_list_nth (rt->history, rt->history_cur - 1);
			if (last == NULL)
			{
				rt->history = NULL;
			} else
				last->next = NULL;
			e2_list_free_with_data (&free);
		}
		e2_list_update_history (path, &rt->history,
			&rt->history_cur, 30, TRUE);
	}
	if (hook)
		//run any change-dir hook function(s)
		e2_hook_list_run (&rt->hook_change_dir, rt->path);
}
/**
 * @brief for a specified pane, open the directory @a number places forward in the history list
 *
 * The last entry in the list will be used if we try to go forward too far
 *
 * @param rt  pointer to pane data struct
 * @param number integer no.of places in the history list to go forward
 * 
 * @return
**/
static void _e2_pane_go_forward (E2_PaneRuntime *rt, gint number)
{
	printd (DEBUG, "_e2_pane_go_forward (rt:,number:%d", number);
	guint old = rt->history_cur;
	guint len = g_list_length (rt->history);
	rt->history_cur += number;
	if (rt->history_cur > g_list_length (rt->history))
		rt->history_cur = len;
	if (old != rt->history_cur)
	{
//		gchar *dir = F_FILENAME_TO_LOCALE (
		gchar *dir =
			g_list_nth_data (rt->history, rt->history_cur - 1);
		_e2_pane_change_dir (rt, dir, FALSE, TRUE);
//		F_FREE (dir);
	}
}
/**
 * @brief goto the next dir in the history list for a specified pane
 *
 * @param rt  pointer to pane data struct, or NULL for active pane
 * 
 * @return
**/
static void e2_pane_go_forward (E2_PaneRuntime *rt)
{
	if (rt == NULL)
		rt = curr_pane;
	_e2_pane_go_forward (rt, 1);
}
/**
 * @brief for a specified pane, open the directory @a number places back in the history list
 *
 * The first entry in the list will be used if we try to go back too far
 *
 * @param rt  pointer to pane data struct
 * @param number integer no.of places in the history list to go back
 * 
 * @return
**/
static void _e2_pane_go_back (E2_PaneRuntime *rt, gint number)
{
	printd (DEBUG, "_e2_pane_go_back (rt:,number:%d", number);
	guint old = rt->history_cur;
	rt->history_cur -= number;
	if (rt->history_cur < 1)
		rt->history_cur = 1;
	if (old != rt->history_cur)
	{
//		gchar *dir = F_FILENAME_TO_LOCALE (
		gchar *dir =
			g_list_nth_data (rt->history, rt->history_cur - 1);
		_e2_pane_change_dir (rt, dir, FALSE, TRUE);
//		F_FREE (dir);
	}
}
/**
 * @brief goto the previous dir in the history list for a specified pane
 *
 * @param rt  pointer to pane data struct, or NULL for current pane
 * 
 * @return
**/
static void e2_pane_go_back (E2_PaneRuntime *rt)
{
	if (rt == NULL)
		rt = curr_pane;
	_e2_pane_go_back (rt, 1);
}
/**
 * @brief for a specified pane, open the parent of the current directory
 *
 * @param rt pointer to pane data struct, or NULL for current pane
 * 
 * @return
**/
static void _e2_pane_go_up (E2_PaneRuntime *rt)
{
	if (rt == NULL)
		rt = curr_pane;
	e2_pane_change_dir (rt, "..");
}
/**
 * @brief show a chosen directory in a pane
 *
 * @param rt pointer to pane data struct, or NULL to determine the pane
 * @param actionrt_data UNUSED
 * @param from pointer to widget where the action was initiated, or NULL
 *
 * @return
**/
void e2_pane_goto_choice (E2_PaneRuntime *rt, gpointer actionrt_data, GtkWidget *from)
{
	//make sure we know the relevant pane runtime and commandline runtime
	E2_CommandLineRuntime *clrt;
	GList *line;
	for (line = app.command_lines; line != NULL ; line = g_list_next (line))
	{
		clrt = line->data;
		if (! clrt->original	//this is a dir line
			&& (GTK_BIN (clrt->cl)->child == from //processing a keybinding, from != NULL
					|| clrt->pane == rt) //processing a mouse-click, rt != NULL
		)
		{
			rt = clrt->pane;
			break;
		}
	}
	if (line == NULL)	//OOPS !!
		return;	//can't do anything

	GtkWidget *entry = GTK_BIN (clrt->cl)->child;
	//create relevant path string
	gchar *dir = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
	if (!g_str_has_prefix (dir, G_DIR_SEPARATOR_S))
	{
		gchar *freeme = dir;
		dir = g_strconcat (rt->path, dir, NULL);
		g_free (freeme);
	}
	e2_fs_get_valid_path (&dir, FALSE);

	gchar *newdir = NULL;
	e2_opendir_dialog_create (dir, &newdir);
	if (newdir != NULL)
	{	//something was selected
		e2_pane_change_dir (rt, newdir);
		g_free (newdir);
	}
	else
	{	//user cancelled
		gtk_widget_grab_focus (entry);
		gtk_editable_select_region (GTK_EDITABLE (entry),
			-1, -1);	//unselect the entry contents
	}
	g_free (dir);
}
/**
 * @brief open the last-used trash directory
 *
 * @return
**/
void _e2_pane_goto_trash (void)
{
	const gchar *tp = e2_utils_get_trash_path ();
	if (tp != NULL)
		e2_pane_change_dir (other_pane, (gchar *) tp);
	else
		e2_output_print_error (_("No trash directory is available"), FALSE);
}
/**
 * @brief switch panes if the specified pane is not the active one
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
static void _e2_pane_focus_action (E2_PaneRuntime *rt)
{
	if (curr_pane != rt)
		e2_pane_activate_other ();
}
/**
 * @brief toggle focus between a dir line and the file pane
 *
 * @param data UNUSED pointer to ??
 * @param arg string with "1" (actually, anything but "2") or "2" which specifies the dir line to focus
 * 
 * @return
**/
static void _e2_pane_focus_dirline_action (gpointer data, gchar *arg)
{
	//decide which dirline
	if (arg != NULL)
	{
		g_strstrip (arg);
		if (!g_str_equal (arg, "2"))	//CHECKME no translation ??
			arg = NULL;
	}
	E2_PaneRuntime *pane = (arg==NULL) ? &app.pane1 : &app.pane2;
	//find the first-registered command-line for the pane
	E2_CommandLineRuntime *cl_rt;
	GList *list;
	for (list = app.command_lines; list != NULL ; list = g_list_next (list))
	{
		cl_rt = (E2_CommandLineRuntime *) list->data;
		if (!cl_rt->original && cl_rt->pane==pane)
			break;
	}
	if (list == NULL)
		return;	//no dirline found for the pane
	if (GTK_WIDGET_HAS_FOCUS (GTK_BIN (cl_rt->cl)->child))
		gtk_widget_grab_focus (curr_view->treeview);
	else
	{
		gtk_editable_set_position (GTK_EDITABLE (GTK_BIN (cl_rt->cl)->child), -1);
		gtk_widget_grab_focus (GTK_BIN (cl_rt->cl)->child);
	}
}
/**
 * @brief select all items in specified pane
 * The pane is made active, if it wasn't already
 * @param rt pointer to pane data struct for the pane whose content is to be selected
 * 
 * @return
**/
static void _e2_pane_select_all (E2_PaneRuntime *rt)
{
	if (rt != curr_pane)
		e2_pane_activate_other ();
	e2_fileview_select_all (NULL, curr_view);
	return;
}
/**
 * @brief invert selection state of all items in active pane
 *
 * This is used for a key-binding
 * 
 * @return
**/
static void _e2_pane_current_invert_all (void)
{
	GtkTreeIter iter;
	GtkTreeModel *mdl = GTK_TREE_MODEL (curr_view->store);
	if (gtk_tree_model_get_iter_first (mdl, &iter))
	{
		GtkTreeSelection *sel = curr_view->selection;
		do
		{
			if (gtk_tree_selection_iter_is_selected (sel, &iter))
				gtk_tree_selection_unselect_iter (sel, &iter);
			else
				gtk_tree_selection_select_iter (sel, &iter);
		} while (gtk_tree_model_iter_next (mdl, &iter));
	}
}
/**
 * @brief toggle selection of all items in active pane
 *
 * This is used for a key-binding
 * 
 * @return
**/
static void _e2_pane_current_select_all (void)
{
	e2_fileview_select_all (NULL, curr_view);
}
/**
 * @brief invert selection state of focused item in active pane
 *
 * This is used for a key-binding
 *
 * @return
**/
static void _e2_pane_current_invert_current (void)
{
	GtkTreeIter iter;
	GtkTreeModel *mdl = GTK_TREE_MODEL (curr_view->store);
	if (gtk_tree_model_iter_nth_child (mdl, &iter, NULL, curr_view->row))
	{
		GtkTreeSelection *sel = curr_view->selection;
		if (gtk_tree_selection_iter_is_selected (sel, &iter))
			gtk_tree_selection_unselect_iter (sel, &iter);
		else
			gtk_tree_selection_select_iter (sel, &iter);
	}
}
/**
 * @brief focus active pane file list
 *
 * This is really intended for the start of a chained a key-binding
 *
 * @return
**/
static void _e2_pane_current_focus (void)
{
	gtk_widget_grab_focus (curr_view->treeview);
}
/**
 * @brief goto top of active pane file list
 *
 * This may be used for a key-binding
 *
 * @return
**/
static void _e2_pane_current_focus_top (void)
{
	if (gtk_tree_model_iter_n_children
			(GTK_TREE_MODEL (curr_view->store), NULL) > 0)
		e2_fileview_focus_row (curr_view, 0, TRUE, TRUE, FALSE, TRUE);
}
/**
 * @brief goto bottom of active pane file list
 *
 * This may be used for a key-binding
 *
 * @return
**/
static void _e2_pane_current_focus_bottom (void)
{
	gint n = gtk_tree_model_iter_n_children
					(GTK_TREE_MODEL (curr_view->store), NULL);
	if (n > 0)
		e2_fileview_focus_row (curr_view, n-1, TRUE, TRUE, FALSE, TRUE);
}
/**
 * @brief open pane other pane directory in this pane
 *
 * @param rt pointer to pane data struct for the pane which is to mirror the other one
 * 
 * @return
**/
static void _e2_pane_mirror (E2_PaneRuntime *rt)
{
	E2_PaneRuntime *other = (rt == &app.pane1) ?
		&app.pane2 : &app.pane1;
	e2_pane_change_dir (rt, other->path);
}
/**
 * @brief helper for toggle display of hidden items
 * @param label action-name label for the pane, _A(10) or _A(11)
 * @param view data struct for the view whose content is to be updated
 * @return
**/
static void _e2_pane__toggle_hidden (FileView *view)
{
	view->show_hidden = !view->show_hidden;
#ifdef E2_NEWFILTER
	e2_fileview_refilter_list (view);
#else
    e2_fileview_refresh_list (view);
#endif
	//toggle the button
	E2_ToggleType num = (view == &app.pane1_view) ?
		E2_TOGGLE_PANE1HIDDEN : E2_TOGGLE_PANE2HIDDEN;
	e2_toolbar_button_toggle (toggles_array[num]);
	return;
}
/**
 * @brief toggle display of hidden items in active pane
 *
 * This is used for a key-binding
 * 
 * @return
**/
static void _e2_pane_current_toggle_hidden (void)
{
	_e2_pane__toggle_hidden (curr_view);
}
/**
 * @brief create and pop up a filters menu for a specified pane
 * The toggle-button state will be changed if appropriate
 * Cannot use a button-click callback to do the menu, as we would 
 * need to connect the same menu to 2 buttons, which is very bad
 * for bar/window refreshes	 
 *
 * @param rt pointer to pane data struct for the pane to be altered
 * @param rt_data UNUSED ptr to data sent to this command
 * @param widget the clicked toolbar item (its child is the widget we use)
 * 
 * @return
**/
static void  _e2_pane_filter_menu_create (E2_PaneRuntime *rt,
	gpointer rt_data, GtkWidget *widget)
{
	FileView *view = (rt == &app.pane1) ? 
		&app.pane1_view : &app.pane2_view;
	GtkWidget *menu = _e2_pane_create_filter_menu (view);
	//to determine the menu's popup position:
	//first, get the button that was pressed
	GtkWidget *button = GTK_BIN (widget)->child;

	GtkAllocation menu_position;
	e2_utils_get_abs_pos (button, &(menu_position.x), &(menu_position.y));
	e2_toobar_menu_position_set (button, menu, &menu_position,
		&rt->toolbar);

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
		(GtkMenuPositionFunc) e2_toolbar_menu_position, &menu_position, 1, 0);
}

  /******************/
 /***** public *****/
/******************/

/**
 * @brief grab pointers to options previously registered
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
void e2_pane_create_option_data (E2_PaneRuntime *rt)
{
	gchar *option_name = (rt == &app.pane1) ?
		"panebar1" : "panebar2" ;	//do not translate
	rt->set = e2_option_get (option_name);
	rt->opt_transparent = e2_option_get ("transparent-dir-links");
	option_name = g_strconcat (rt->name,"-use-startup-dir",NULL);
	rt->opt_use_startup = e2_option_get (option_name);
	g_free (option_name);
	option_name =  g_strconcat(rt->name,"-use-startup-dir-startup-dir",NULL);
	rt->opt_startup = e2_option_get (option_name);
	g_free (option_name);
	//create dependency
	rt->opt_startup->depends = rt->opt_use_startup->name;
}
/**
 * @brief create file pane and all its contents
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
void e2_pane_create (E2_PaneRuntime *rt)
{
	gint num = (rt == &app.pane1) ? 1 : 2;
	rt->name = g_strdup_printf ("pane%d", num);
	rt->pane_sw = NULL;

	e2_pane_create_option_data (rt);

	e2_cache_str_register (rt->name, &rt->path, G_DIR_SEPARATOR_S);
	//do we need to over-ride the cached startup dir ?
	gchar *panedir = (num == 1) ? e2_cl_options.pane1_path :
		e2_cl_options.pane2_path ;
	gchar *freeme = NULL;
	if (panedir != NULL)
		panedir = freeme = F_FILENAME_FROM_LOCALE (panedir);
	if (panedir == NULL 
		&& e2_option_bool_get_direct (rt->opt_use_startup))
			panedir = _e2_option_str_get (rt->opt_startup);
	if (panedir != NULL)
	{
		g_free (rt->path);
		rt->path = g_strdup (panedir);
	}
	if (freeme != NULL)
		g_free (freeme);
	//make sure initial path is valid
	e2_fs_get_valid_path (&(rt->path), TRUE);

	E2_BarType thisbar = (num == 1) ? E2_BAR_PANE1 : E2_BAR_PANE2;
	e2_toolbar_initialise (thisbar);

	//register pane-specific actions (all name strings freed in action creation function)
	gchar *pstr = (num == 1) ? _A(10) : _A(11);
	gchar *action_name = g_strconcat (pstr,".",_A(39), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_focus_action, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(23), NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_COMMAND_LINE,
		e2_command_line_directory_cb, "dir line", FALSE,  //internal default name, not translated
		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_ACCEL, rt);
	action_name = g_strconcat (pstr,".",_A(42), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_go_forward, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(41), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_go_back, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(43), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_go_up, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(56), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_goto_choice, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(51),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_mirror, rt, FALSE);
	action_name = g_strdup ((num == 1) ?
		toggles_array [E2_TOGGLE_PANE1HIDDEN] : toggles_array [E2_TOGGLE_PANE2HIDDEN]);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane__toggle_hidden, rt->view, FALSE);
	action_name = g_strconcat (pstr,".",_A(82),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_select_all, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(20), NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_BOOKMARKS, e2_bookmark_open,
		GINT_TO_POINTER (num), TRUE, E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_MENU, NULL);
	action_name = g_strdup ((num == 1) ?
		toggles_array [E2_TOGGLE_PANE1FILTERS] : toggles_array [E2_TOGGLE_PANE2FILTERS]);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_pane_filter_menu_create, rt, FALSE);

	//after the actions are in place, fill in the pane contents
	e2_pane_create_part (rt);
}
/**
 * @brief part of pane creation, used also in re-creation
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
void e2_pane_create_part (E2_PaneRuntime *rt)
{
	g_hook_list_init (&rt->hook_change_dir, sizeof (GHook));

	gchar *history_name = g_strconcat (rt->name, "-history", NULL);
	e2_cache_list_register (history_name, &rt->history);
	if (rt->history != NULL)  //if history list found in cache
		rt->history_cur = g_list_length (rt->history);
	g_free (history_name);

	rt->pane_sw = e2_fileview_create_list (rt->view);
	rt->outer_box = gtk_hbox_new (FALSE, 0);
	rt->inner_box = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start_defaults (GTK_BOX (rt->inner_box), rt->pane_sw);
	gtk_box_pack_start_defaults (GTK_BOX (rt->outer_box), rt->inner_box);

	gtk_widget_show (rt->inner_box);
	gtk_widget_show (rt->outer_box);

	e2_toolbar_create (&rt->toolbar);

	if (e2_option_bool_get_direct (rt->toolbar.show))
	{
		//show full/split window button according to cached pane ratio
		if (rt == &app.pane1 && app.window.panes_paned_ratio > 0.999)
			e2_toolbar_toggle_button_set_state
				(toggles_array [E2_TOGGLE_PANE1FULL], TRUE);
		else if (rt == &app.pane2 && app.window.panes_paned_ratio < 0.001)
			e2_toolbar_toggle_button_set_state
				(toggles_array [E2_TOGGLE_PANE2FULL], TRUE);
		//show (un)hide toggle buttons which match corresponding cached state
		E2_ToggleType num = (rt == &app.pane1) ?
			E2_TOGGLE_PANE1HIDDEN : E2_TOGGLE_PANE2HIDDEN;
		e2_toolbar_toggle_button_set_state (toggles_array [num], rt->view->show_hidden);
		num = (rt == &app.pane1) ?
			E2_TOGGLE_PANE1FILTERS : E2_TOGGLE_PANE2FILTERS;
		gboolean filtered =
			(  rt->view->name_filter.active
			|| rt->view->size_filter.active
			|| rt->view->date_filter.active);
		//this needs to be inverted to show correct button
		e2_toolbar_toggle_button_set_state (toggles_array [num], !filtered);
		//set initial filter tooltip
		if (filtered)
			e2_toolbar_toggle_filter_button (rt->view);
	}
}
/**
 * @brief destroy pane data
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
/*static void _e2_pane_destroy (E2_PaneRuntime *rt)
{
	gchar *history_name = g_strconcat (rt->name, "-history", NULL);
	e2_cache_unregister (history_name);
	g_free (history_name);
//	e2_toolbar_destroy (&rt->toolbar);
	//pane toolbar may be outside rt->outer_box, so destroy it independently
	gtk_widget_destroy (rt->toolbar.toolbar_container);
//	gtk_widget_destroy (rt->pane_sw);
	gtk_widget_destroy (rt->outer_box);
	g_hook_list_clear (&rt->hook_change_dir);
}*/
/**
 * @brief destroy and re-create specified pane
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
/*void e2_pane_recreate (E2_PaneRuntime *rt)
{
	_e2_pane_destroy (rt);
	e2_pane_create_part (rt);
}*/
/**
 * @brief register pane-related actions that do have a pane-specifc action
 *
 * Actions with pane-specific function are registered in e2_pane_create()
 *
 * @return
**/
void e2_pane_actions_register (void)
{  //these strings are freed during the registration process
	gchar *action_name = g_strconcat(_A(9),".",_A(42),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_go_forward, NULL, FALSE);
	action_name = g_strconcat(_A(9),".",_A(41),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_go_back, NULL, FALSE);
	action_name = g_strconcat(_A(9),".",_A(43),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_go_up, NULL, FALSE);
	action_name = g_strconcat(_A(12),".",_A(56),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_goto_choice, NULL, FALSE);
	action_name = g_strconcat(_A(9),".",_A(75),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_toggle_hidden, NULL, FALSE);
	action_name = g_strconcat(_A(9),".",_A(49),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_invert_all, NULL, FALSE);
	action_name = g_strconcat(_A(9),".",_A(83),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_invert_current, NULL, FALSE);
	action_name = g_strconcat(_A(9),".",_A(82),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_select_all, NULL, FALSE);
	action_name = g_strconcat(_A(9),".",_A(76),NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM, e2_context_menu_show_menu_action, NULL, TRUE,
		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_TOOLBAR, NULL);
	action_name = g_strconcat(_A(9),".",_A(39),NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_focus, NULL, FALSE,
		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_TOOLBAR, NULL);
	action_name = g_strconcat(_A(9),".",_A(45),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_focus_top, NULL, FALSE);
	action_name = g_strconcat(_A(9),".",_A(44),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_focus_bottom, NULL, FALSE);
	action_name = g_strconcat(_A(6),".",_A(84),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_goto_trash, NULL, FALSE);
	action_name = g_strconcat(_A(4),".",_A(40),NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_focus_dirline_action, NULL, TRUE,
		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_TOOLBAR, NULL);
	action_name = g_strconcat(_A(9),".",_A(21),NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_filters_show, NULL, FALSE,
		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_ACCEL, NULL);
}
/**
 * @brief register pane-related config options
 * Panebar default contents are set here, too
 * @param num integer pane number (1 or 2)
 *
 * @return
**/
void e2_pane_options_register (gint num)
{
	e2_toolbar_panebar_register (num);	//setup the pane bar

	gint i; gchar *option_name, *desc, *tip, *grey_me;
	E2_OptionFlags flags;
	desc = (num == 1) ? _A(10) : _A(11);
//	group_name = g_strconcat (parent, ".", _C(2), ":", desc, NULL);    // _("panes.columns:pane N"
	gchar *group_name = g_strconcat ((num == 1) ? _C(28) : _C(30), ":", _C(2), NULL);    // _("paneN:columns"
	for (i = 0; i < MAX_COLUMNS; i++)
	{
		option_name = g_strdup_printf ("pane%d-show-column%d", num, i);
		desc = g_strdup_printf (_("show %s column"), gettext(e2_all_columns[i].title));
		tip = g_strdup_printf (_("This causes the the named column to be displayed in pane %d (duh ...)"), num);
		flags = E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREENAME | E2_OPTION_FLAG_FREEDESC
			| E2_OPTION_FLAG_FREETIP | E2_OPTION_FLAG_BUILDPANES;
		if (i > 0)
			grey_me = (num == 1) ? "!pane1-uses-other" : "!pane2-uses-other";
		else
		{
			flags |= E2_OPTION_FLAG_FREEGROUP;
			//disable hiding of col 0 = filename
			grey_me = g_strconcat ("!",option_name,NULL); //a 'depends' entry = !<self> disables any change
		}
		e2_option_bool_register (option_name, group_name, desc, tip, grey_me,
		TRUE, flags);
	}

	if (num == 1)
		e2_toolbar_options_register (E2_BAR_PANE1);  //set general toolbar options
	else
		e2_toolbar_options_register (E2_BAR_PANE2);
	//hack to make general options show after the panes, in basic mode
	if (num == 2)
	{
		for (num = 1; num < 3; num++)
		{
			option_name = g_strdup_printf ("pane%d-use-startup-dir", num);
		//	group_name = g_strconcat (parent, ":", _C(36), NULL);    // _("panes:startup"
			group_name = g_strconcat (_C(16), ":", _C(36), NULL);    // _("general:startup"
			desc = g_strdup_printf (_("at startup, show a specific directory in pane %d"), num);
			e2_option_bool_register (option_name, group_name,
				desc, _("This causes the directory named below, instead of the last-opened directory, to be shown at session start"),
				NULL, FALSE,
				E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREEGROUP 
				| E2_OPTION_FLAG_FREENAME | E2_OPTION_FLAG_FREEDESC );
			option_name = g_strconcat (option_name,"-startup-dir", NULL);
			desc = g_strdup_printf (_("pane %d startup directory:"), num);
		//FREEME = non-constant name, desc
			e2_option_str_register (option_name, group_name,
				desc,  _("This is the directory to show in this pane, at session start"),
				"",  //= no dependency, changed to rt->opt_use_startup->name after pane rt is created
				(num == 1) ? "~" : G_DIR_SEPARATOR_S,   //pane1 default = home, pane2 default = root directory
				E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREENAME | E2_OPTION_FLAG_FREEDESC);
		}
	}
}
