/* $Id: e2_task.h 1869 2006-04-27 01:58:02Z tpgww $
 *
 * Copyright (C) 2003-2006 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef _E2_TASK_H
#define _E2_TASK_H

#include "emelfm2.h"
#include "e2_permissions_dialog.h"

// TEMP_FAILURE_RETRY for systems which do not have it
#ifndef TEMP_FAILURE_RETRY
#define TEMP_FAILURE_RETRY(expr) \
    ({ long int _res; \
       do _res = (long int) (expr); \
       while (_res == -1L && errno == EINTR); \
       _res; })
#endif

//this is the deepest level of nested dirs that will be processed
#define E2_DIRNEST_LIMIT 32

#ifndef ALLPERMS
#define ALLPERMS (S_ISUID|S_ISGID|S_ISVTX|S_IRWXU|S_IRWXG|S_IRWXO)
#endif

typedef enum
{
	E2_TASK_COPY,
	E2_TASK_COPYAS,
	E2_TASK_MOVE,
	E2_TASK_MOVEAS,
	E2_TASK_LINK,
	E2_TASK_LINKAS,
	E2_TASK_DELETE,
	E2_TASK_RENAME,
	E2_TASK_TRASH,
	E2_TASK_TRASHEMPTY,
	E2_TASK_CHMOD,
	E2_TASK_CHOWN,
	E2_TASK_INFO,
	E2_TASK_VIEW,
	E2_TASK_EDIT,
	E2_TASK_OPEN,
	E2_TASK_OPENWITH,
	E2_TASK_DROP,
	E2_TASK_LAST	//this is just a code for plugins etc
} E2_TaskType;

#ifdef E2_VFS
typedef struct _E2_RefreshData
{
	guint task_id;	//which task
	time_t timelimit;	//if not finished by this, abort the task
	gboolean result;	//result from the function performing the task
	gboolean ready;	//TRUE when ok to do the reftesh (the task has completed successfully)
	gchar *dir;	//the dir to be refreshed, utf8 string with trailer
	GtkWidget *initiator;	//the widget which was acivated to initiate the task
	E2_TaskType tasktype;	//what type of task (supports history)
} E2_RefreshData;
#endif

typedef struct 
{
	gchar *path;
	mode_t mode;
} E2_DirEnt;

typedef struct 
{
	gboolean sametime;	//TRUE to preserve a/ctime of copied files
	gboolean continued_after_problem;	//TRUE when a problem is found, but the walk is not aborted
	gint oldroot_len;	//length of source item path, less its last element
	gchar *newroot;	//destination item path, less its last element, no trailing /, localised string
	GList *dirdata;	//list of directories copied
} E2_CopyData;

typedef struct
{
	gboolean continued_after_problem;
	E2_RecurseType scope;
	mode_t setmask;
	mode_t clearmask;
	GList *dirdata;
} E2_ChmodData;

typedef struct
{
	gboolean continued_after_problem;
	uid_t new_uid;
	gid_t new_gid;
	GList *dirdata;
} E2_ChownData;
	   
GList *open_history;  //history list for open-with dialog

gboolean e2_task_do_task (E2_TaskType func, gchar *slocal, gchar *dlocal,
	   gboolean preserve,
#ifdef E2_INCLIST
		GtkTreeRowReference *ref);
#else
		GtkTreePath *path);
#endif

gboolean e2_task_error (gchar *msg);
gboolean e2_task_error_local (const gchar *format, gchar *local);
#ifdef LATE_DELETE
gchar *e2_task_tempname (gchar *path);
#endif

gboolean e2_task_copy (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_copy_as (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_move (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_move_as (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_rename (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_trashit (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_delete (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_symlink (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_symlink_as (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_permissions (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_ownership (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_file_info (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_sync_dirs (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_view (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_edit (gpointer action_data, gpointer rt_data, GtkWidget *widget);
//gboolean __e2_task_toggle_tag (void);
gboolean e2_task_drop (E2_TaskType func, guchar *uri_list, gchar *dest_dir);
gboolean e2_task_refresh (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_command (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_open (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_open_with (gpointer action_data, gpointer rt_data, GtkWidget *widget);
gboolean e2_task_open_in_other_pane (gpointer action_data, gpointer rt_data,
	GtkWidget *widget);
gboolean e2_task_configure (GtkWidget *widget, gchar *page);
gboolean e2_task_configure_default (GtkWidget *widget);
gboolean e2_task_find (gpointer action_data, gpointer rt_data, GtkWidget *widget);

gboolean e2_task_backend_copy (gchar *src, gchar *dest, gboolean preserve);
gboolean e2_task_backend_move (gchar *src, gchar *dest);
gboolean e2_task_backend_link (gchar *target, gchar *name);
gboolean e2_task_backend_rename (gchar *oldsrc, gchar *newsrc);
gboolean e2_task_backend_delete (gchar *path);
gboolean e2_task_backend_chmod (gchar *path, gchar *mode, E2_RecurseType recurse);
gboolean e2_task_backend_chown (gchar *path, uid_t owner_id, gid_t group_id,
	gboolean recurse);
gboolean e2_task_backend_open (FileInfo *info);
gboolean e2_task_backend_view (gchar *filename);
void e2_task_actions_register (void);

#endif //ndef _E2_TASK_H
