/* $Id: e2_combobox.h 1789 2006-04-02 07:27:15Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file e2_combobox.h 
 * @brief GtkComboBox utilities header
 *
 * This is the header file for the GtkComboBox utilities.
 **/

#ifndef E2_COMBOBOX_H
#define E2_COMBOBOX_H

/**
 * @brief These flags influence the GtkComboBox creation in several ways.
 * 
 * These flags influence the GtkComboBox creation in several ways
 * when creating them with e2_combobox_get() or e2_combobox_add().
 **/
typedef enum
{
	E2_COMBOBOX_ALLOW_DOUBLE=1<<0,
	/**< Allow the same history entries to be added several times. */
	E2_COMBOBOX_MENU_STYLE=1<<1,
	/**< The drop down widget is a menu. */
	E2_COMBOBOX_LIST_STYLE=1<<2,      
	/**< The drop down widget is a list. */
	E2_COMBOBOX_HAS_ENTRY=1<<3,
	/**< This GtkComboBox has an entry. You should use list style then! */
	E2_COMBOBOX_CYCLE_HISTORY=1<<4,
	/**< If the GtkComboBox has an entry, the history will (if necessary) be
	 * cycled upon pressing GDK_UP or GDK_DOWN. */
	E2_COMBOBOX_FOCUS_ON_CHANGE=1<<5,
	/**< If the GtkComboBox emits the changed signal, grab the focus for it. */
	E2_COMBOBOX_NO_AUTO_HISTORY=1<<6,
	/**< TBA */
} E2_ComboBoxFlags;

//callbacks
void e2_combobox_activated_cb (GtkWidget *entry, gpointer data);

//public
//void e2_combobox_select_last (GtkWidget *combo);
void e2_combobox_block_changed (GtkWidget *combo);
void e2_combobox_unblock_changed (GtkWidget *combo);
void e2_combobox_set_active (GtkWidget *combo, gint num);
gchar *e2_combobox_get_active (GtkWidget *combo);
void e2_combobox_append_history (GtkWidget *combo, GList *list);
void e2_combobox_append_history_strv (GtkWidget *combo, gchar **strv);
gint e2_combobox_last_index (GtkComboBox *combo);
gchar *e2_combobox_last_text (GtkComboBox *combo);
GtkWidget *e2_combobox_get (gpointer func, gpointer data, E2_ComboBoxFlags flags);
GtkWidget *e2_combobox_add (GtkWidget *box, gboolean expand, guint padding,
	gpointer func, gpointer data, E2_ComboBoxFlags flags);

#endif //ndef E2_COMBOBOX_H
